# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from __future__ import division
import operator
from math import ceil, log10
import copy

class Coord(tuple):
	def __add__(self, other):
		return self.__class__(map(operator.add, self, other))

class Attributes:
	def __init__(self,parent=None,**kwargs):
		self.__content = {}
		if parent != None:
			self.extend(**(parent.getDict()))
		self.extend(**kwargs)
	def extend(self,**attributes):
		self.__content.update(attributes)
		for (k, v) in attributes.iteritems():
			setattr(self, k, v)
	def update(self,**attributes):
		for (k, v) in attributes.iteritems():
			if not k in self.__content.keys():
				raise "Attribute “%s” unknown."%k
			self.__content.update({k:v})
			setattr(self, k, v)
	def getDict(self):
		return self.__content.copy()
	def copy(self):
		return Attributes(**(copy.deepcopy(self.__content)))

class ziktRenderException(BaseException):
	def __init__(self,msg):
		self.msg = msg
	def __str__(self):
		return repr(self.msg)
class ziktTableException(BaseException):
	def __init__(self,msg):
		self.msg = msg
	def __str__(self):
		return repr(self.msg)

def normalizeAngle(angle):
	return (360 + (angle % 360)) % 360

def angleIsNorthernHemisphere(angle):
	return (normalizeAngle(angle) <= 180)

def angleIsSouthernHemisphere(angle):
	return (normalizeAngle(angle) >= 180)

def angleIsWesternHemisphere(angle):
	return angleIsSouthernHemisphere(angle+90)

def angleIsEasternHemisphere(angle):
	return angleIsNorthernHemisphere(angle+90)

def decimalCeil(v):
	if v >= 1:
		v = ceil(float(v))
		digits=ceil(log10(v+0.1))
		return ceil(v*pow(10,-digits+1))*pow(10,digits-1)
	if v > 0 and v < 1:
		k = 1/v
		zeros=ceil(log10(k+0.1))
		return decimalCeil(v*pow(10,zeros))/pow(10,zeros)
	if v == 0:
		return 0
	else:
		return -decimalCeil(-v)
	



# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from helper import ziktRenderException, Attributes
from helper import angleIsNorthernHemisphere,angleIsSouthernHemisphere,angleIsWesternHemisphere,angleIsEasternHemisphere
from helper import normalizeAngle
from math import radians, cos
from zikt.tikzrender import TikZRenderer
from zikt.tables_tests import getBasicTable
from zikt.tables import FloatTable


	

class PieChart(TikZRenderer):
	presettings =  {
		'percent': Attributes(
			areaStyles=["sectorone","sectortwo","sectorthree","sectorfour","sectorfive","sectorsix","sectorseven","sectoreight","sectornine","sectorten","sectoreleven","sectortwelve"],
			areaLabelStyles=["sectorlabelone","sectorlabeltwo","sectorlabelthree","sectorlabelfour","sectorlabelfive","sectorlabelsix","sectorlabelseven","sectorlabeleight","sectorlabelnine","sectorlabelten","sectorlabeleleven","sectorlabeltwelve"],
			areaOuterLabelStyles=["sectorlabelone","sectorlabeltwo","sectorlabelthree","sectorlabelfour","sectorlabelfive","sectorlabelsix","sectorlabelseven","sectorlabeleight","sectorlabelnine","sectorlabelten","sectorlabeleleven","sectorlabeltwelve"],
			areaConnectedLabelStyles=["sectorlabelone","sectorlabeltwo","sectorlabelthree","sectorlabelfour","sectorlabelfive","sectorlabelsix","sectorlabelseven","sectorlabeleight","sectorlabelnine","sectorlabelten","sectorlabeleleven","sectorlabeltwelve"],
			areaConnectorStyles=["sectorconnectorone","sectorconnectortwo","sectorconnectorthree","sectorconnectorfour","sectorconnectorfive","sectorconnectorsix","sectorconnectorseven","sectorconnectoreight","sectorconnectornine","sectorconnectorten","sectorconnectoreleven","sectorconnectortwelve"],
			circleStyle="outercircle",
			delimiterStyle="delimiter",
			clockwise=True,
			startangle=180,
			radius=2,
			areaLabelCenterDistance=1.2,
			showValuePercentage = False,
			showValueOutsideThresholdPercentage = 20,
			areaOuterLabelStartDistance = 1.7,
			areaOuterLabelKinkDistance = 2.5,
			areaOuterLabelOffsetMin = 1,
			areaOuterLabelOffsetMax = 2.5,
			baseunit='cm',
			additionalSectorStyle="",
			areaLabel='',
			areaLabel_small=u'',
			connectedLabel=u'',
			connectedLabel_small=u'{valuePercentage:.2f}\,\%',
		)
	}
	
	def __init__(self,preset="percent", row=None, printComments=True, **kwargs):
		self.attributes = PieChart.presettings[preset]
		self.attributes.extend(**kwargs)
		self.row = row
		self.printComments = printComments
		self.attributeDict = self.attributes.getDict()
	
	class _Sector:
		def __init__(self,
					renderer,index,value,lastangle,total,ratioIndexOffset,
					areaStyles,
					areaLabelStyles,
					areaOuterLabelStyles,
					areaConnectedLabelStyles,
					areaConnectorStyles,
					title=None,
					):
			self.renderer = renderer
			self.value=value
			self.valueRatio = value / total
			self.valuePercentage = self.valueRatio * 100
			self.width = 360.0 * self.valueRatio
			self.index = index
			self.ratioIndexStart = ratioIndexOffset
			self.ratioIndexCenter = self.ratioIndexStart + self.valueRatio / 2
			self.ratioIndexEnd = self.ratioIndexStart + self.valueRatio
			self.startangle = lastangle
			if renderer.attributes.clockwise:
				self.endangle = self.startangle - self.width
				self.midangle = self.startangle - self.width / 2
			else:
				self.endangle = self.startangle + self.width
				self.midangle = self.startangle + self.width / 2
			self.attributesDict = renderer.attributes.getDict()
			self.attributesDict.update({
				'title' : title,
				'value' : self.value,
				'valueRatio' : self.valueRatio,
				'valuePercentage' : self.valuePercentage,
				'width' : self.width,
				'index' : self.index,
				'startangle' : self.startangle,
				'endangle' : self.endangle,
				'midangle' : self.midangle,
				'ratioIndexStart' : self.ratioIndexStart,
				'ratioIndexCenter' : self.ratioIndexCenter,
				'ratioIndexEnd' : self.ratioIndexEnd,
				'sectorclass':areaStyles[index],
				'arealabelclass':areaLabelStyles[index],
				'areaouterlabelclass':areaOuterLabelStyles[index],
				'connectedlabelclass':areaConnectedLabelStyles[index],
				'connectorclass':areaConnectorStyles[index],
			})
			self.attributesDict.update({
				'arealabel':self.renderer.attributes.areaLabel.format(**self.attributesDict),
				'arealabel_small':self.renderer.attributes.areaLabel_small.format(**self.attributesDict),
				'connectedlabel':self.renderer.attributes.connectedLabel.format(**self.attributesDict),
				'connectedlabel_small':self.renderer.attributes.connectedLabel_small.format(**self.attributesDict),
			})
		def isNorthernHemisphere(self):
			return angleIsNorthernHemisphere(self.midangle)
		def isSouthernHemisphere(self):
			return angleIsSouthernHemisphere(self.midangle)
		def isWesternHemisphere(self):
			return angleIsWesternHemisphere(self.midangle)
		def isEasternHemisphere(self):
			return angleIsEasternHemisphere(self.midangle)
		def getOuterLabelOffset(self):
			d = (self.renderer.attributes.areaOuterLabelOffsetMax - self.renderer.attributes.areaOuterLabelOffsetMin)
			o = self.renderer.attributes.areaOuterLabelOffsetMin
			return (1 - abs(cos(radians(normalizeAngle(self.midangle))))) * d + o
		def getTikzForSectorArea(self):
			#return ("\t\\path[{sectorclass}]").format(**self.attributesDict)
			add = "" if self.renderer.attributes.additionalSectorStyle == "" else "{"+self.renderer.attributes.additionalSectorStyle+"}"
			return ("\\path[{sectorclass},"+add+"] (0,0) -- ({startangle}:{radius}) arc ({startangle}:{endangle}:{radius}) -- cycle;").format(**self.attributesDict)
		def getTikzForSectorDelimiter(self):
			return ("\\path[{delimiterStyle}] (0,0) -- ({startangle}:{radius});").format(**self.attributesDict)
		def getTikzForSectorLabel(self):
			label = '{arealabel}' if self.valuePercentage >= self.renderer.attributes.showValueOutsideThresholdPercentage else '{arealabel_small}'
			label_class = '{areaouterlabelclass}' if self.valuePercentage >= self.renderer.attributes.showValueOutsideThresholdPercentage else '{arealabelclass}'
			return ("\\path (0,0) ({midangle}:{areaLabelCenterDistance}) node["+label_class+"] {{"+label+"}};").format(**self.attributesDict)
		def getTikzForConnectedLabel(self):
			label = ('{connectedlabel}' if self.valuePercentage >= self.renderer.attributes.showValueOutsideThresholdPercentage else '{connectedlabel_small}').format(**self.attributesDict)
			if label == "":
				return None
			return ("\\path[{connectorclass}] (0,0) ({midangle}:{areaOuterLabelStartDistance}) -- ({midangle}:{areaOuterLabelKinkDistance}) -- ++({endsign}{enddistance},0) node (l) [{connectedlabelclass}, anchor=south {firstanchor}, node distance=0cm] {{{label}}} -- (l.south {secondanchor});").format(**dict(self.attributesDict.items()+{
				'firstanchor':'west' if self.isEasternHemisphere() else 'east',
				'secondanchor':'east' if self.isEasternHemisphere() else 'west',
				'endsign':'+' if self.isEasternHemisphere() else '-',
				'enddistance':str(self.getOuterLabelOffset())+self.renderer.attributes.baseunit,
				'label':label
			}.items()))
			#TODO mach noch das label unter der linie
		
		
	
	def render(self,table):
		result = []
		if self.row == None:
			for row in table:
				result.append(self.renderSingleRow(table,row))
		else:
			result.append(self.renderSingleRow(table,table.rows[self.row]))
		return u'\n'.join(result)
	
	def renderSingleRow(self,table,row):
		# initialize
		total = 0.0
		values = []
		sectors=[]
		titles=[]
		areaStyles=[]
		areaLabelStyles=[]
		areaOuterLabelStyles=[]
		areaConnectedLabelStyles=[]
		areaConnectorStyles=[]
		i=0
		styleOffset = -1
		#TODO nur spezifizierte Datenzellen verwenden und das Überspringen von Null parametrisierbar machen
		for cell in row:
			i += 1
			try:
				f = float(cell)
				if f == 0:
#					styleOffset += 1
					continue
				values.append(f)
				titles.append(table.colHeader[i-1] if table.hasColHeader else None)
				areaStyles.append(self.attributes.areaStyles[(i + styleOffset)%len(self.attributes.areaStyles)])
				areaLabelStyles.append(self.attributes.areaLabelStyles[(i + styleOffset)%len(self.attributes.areaLabelStyles)])
				areaOuterLabelStyles.append(self.attributes.areaOuterLabelStyles[(i + styleOffset)%len(self.attributes.areaOuterLabelStyles)])
				areaConnectedLabelStyles.append(self.attributes.areaConnectedLabelStyles[(i + styleOffset)%len(self.attributes.areaConnectedLabelStyles)])
				areaConnectorStyles.append(self.attributes.areaConnectorStyles[(i + styleOffset)%len(self.attributes.areaConnectorStyles)])
				total += f
			except ValueError, e:
				raise ziktRenderException("Minimum one cell does not contain a valid number. (%s)" % str(e))
		if total == 0:
			raise ziktRenderException("The sum of all values is 0.")
		lastangle = self.attributes.startangle
		ratioIndexOffset = 0
		for i in range(len(values)):
			sectors.append(PieChart._Sector(
				self,
				i,
				values[i],
				lastangle,
				total,
				title = titles[i],
				ratioIndexOffset = ratioIndexOffset,
				areaStyles = areaStyles,
				areaLabelStyles = areaLabelStyles,
				areaOuterLabelStyles = areaOuterLabelStyles,
				areaConnectedLabelStyles = areaConnectedLabelStyles,
				areaConnectorStyles = areaConnectorStyles,
			))
			lastangle = sectors[i].endangle
			ratioIndexOffset = sectors[i].ratioIndexEnd
		self.attributes.extend(total=total, items=len(sectors))
			
		# building output
		result = []
		
		# draw sector areas
		self.printComment(u"\t",u"Drawing the sector areas",result)
		for sector in sectors:
			result.append(u"\t"+sector.getTikzForSectorArea())
			
		# draw sector labels
		for sector in sectors:
			result.append(u"\t"+sector.getTikzForSectorLabel())
		
		# draw sector delimiters
		if self.attributes.delimiterStyle != None:
			for sector in sectors:
				result.append(u"\t"+sector.getTikzForSectorDelimiter())
		
		# draw surrounding circle
		if self.attributes.circleStyle != None:
			result.append(u"\t\\path[{circleStyle}] (0,0) circle ({radius});".format(**self.attributeDict))
		
		# draw connected labels
		for sector in sectors:
			l = sector.getTikzForConnectedLabel()
			if l != None:
				result.append(u"\t"+sector.getTikzForConnectedLabel())
		
		#TODO mach noch das outer label
		
		return u"\n".join(result)

if __name__ == "__main__":
	c =  PieChart(ordinateLength=6,stackmode = True)
	print c.render(table = FloatTable(getBasicTable()))


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''


from zikt.cschart import MultipleValueCoordinateChart
from zikt.helper import Attributes
from zikt.painter import BarPainter
from zikt.tables_tests import getBasicTable
from zikt.tables import FloatTable

class StackedBarChart(MultipleValueCoordinateChart):
	base = Attributes(
		#TODO: high: schaue of diese parents bei den Attributen nicht weg können
		parent=MultipleValueCoordinateChart.base,
		painter=[BarPainter],
		#
		upperLabelString=u"{value}",
		upperLabelStyle=u"",
		lowerLabelString=u"",
		lowerLabelStyle=u"",
		centerLabelString=u"",
		centerLabelStyle=u"",
		#
		stackmode = 'onTopOfZero',
		#
		barStyle = u"ziktStackedBarStyle",
		additionalBarStyle = u"color={outlineColor},fill={fillColor}",
		barWidthRatio = 0.6,
		#
		abscissaTickStyle=None,
		abscissaJutStyle=None,
		abscissaTickLabelString=u"",
	)
	presettings =  {
		'base': base,
	}
	
	def __init__(self,preset="base", printComments=True, **kwargs):
		MultipleValueCoordinateChart.__init__(self)
		self.attributes = StackedBarChart.presettings[preset].copy()
		self.attributes.extend(**kwargs)
		self.printComments = printComments
		self.attributeDict = self.attributes.getDict()
		

if __name__ == "__main__":
	c =  StackedBarChart(ordinateLength=6,stackmode = "onTopOfZero")
	print c.render(table = FloatTable(getBasicTable()))


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''
from __future__ import division
from zikt.helper import ziktRenderException, Attributes, decimalCeil
from cs import TikZCoordinateSystem


class BarChart(TikZCoordinateSystem):
	preset_bar = Attributes(
			parent=TikZCoordinateSystem.empty_preset,
			#
			abscissaJutStyle=None,
			#
			fallbackOrdinateLength=5,
			#
			barWidth=1,
			startMargin=0.5,
			midMargin=0.6,
			endMargin=0.2,
			barMargin=0.3,
			#
			barStyleMode='single', #single or multiple
			barStyle="catsvBarChartBar",	barStyles=["catsvBarChartBarOne","catsvBarChartBarTwo","catsvBarChartBarThree","catsvBarChartBarFour","catsvBarChartBarFive","catsvBarChartBarSix","catsvBarChartBarSeven","catsvBarChartBarEight","catsvBarChartBarNine","catsvBarChartBarTen","catsvBarChartBarEleven","catsvBarChartBarTwelve"],
			#
			additionalBarStyle='',
			additionalOuterLowerLabelStyle='',
			additionalInnerLowerLabelStyle='',
			additionalCenterLabelStyle='',
			additionalInnerUpperLabelStyle='',
			additionalOuterUpperLabelStyle='',
			#
			outerLowerLabelStyle=u'catsvBarChartOuterLowerLabel',
			innerLowerLabelStyle=u'catsvBarChartInnerLowerLabel',
			centerLabelStyle=[u'catsvBarChartCenterLabelHorizontal',u'catsvBarChartCenterLabelVertical'],
			innerUpperLabelStyle=u'catsvBarChartInnerUpperLabel',
			outerUpperLabelStyle=u'catsvBarChartOuterUpperLabel',
			#
			groupUpperLabelStyle=u'catsvBarChartGroupUpperLabel',
			groupLowerLabelStyle=u'catsvBarChartGroupLowerLabel',
			#
			additionalGroupUpperLabelStyle='',
			additionalGroupLowerLabelStyle='',
			#
			outerLowerLabelString=u'{n}',
			innerLowerLabelString=u'',
			centerLabelString=u'',
			innerUpperLabelString=u'',
			outerUpperLabelString=u'',
			#
			groupUpperLabelString=u'',
			groupLowerLabelString=u'',
			#
			flipLowerLabelsWhenNegative=False,
			flipUpperLabelsWhenNegative=False, 
			flipOuterLabelsWhenNegative=True,
			flipInnerLabelsWhenNegative=True, 
			valueConversion='int',#'int','round' or a callable (lambda)
			#
			#TODO fullValue
			#
			barLengthThreshold=1,
			flipLowerAboveWhenThreshold=False,
			flipUpperAboveWhenThreshold=True,
			flipCenterAboveWhenThreshold=False,
			flipLowerBelowWhenThreshold=False,
			flipUpperBelowWhenThreshold=False,
			flipCenterBelowWhenThreshold=False,
		)
	preset_groupedbar = Attributes(
			parent=preset_bar,
			#
			barStyleMode='multiple',
			barWidth=0.5,
			midMargin=0.7,
			barMargin=0.1,
			#
			outerLowerLabelString='',
			innerUpperLabelString='{v}\,\%',
			groupLowerLabelString='{n}',
			#innerUpperLabelStyle='catsvBarChartInnerUpperLabelRotated',
		)
	presettings =  {
		'bar': preset_bar,
		'groupedbar' : preset_groupedbar,
	}
	
	def __init__(self,preset="bar", printComments=True, **kwargs):
		self.attributes = BarChart.presettings[preset]
		self.attributes.extend(**kwargs)
		self.printComments = printComments
		self.attributeDict = self.attributes.getDict()
		
		
	def drawBar(self,index,firstValue,secondValue, percentage,maxValue,minValue,maxOrdinateValue,minOrdinateValue,name,abscissaLeftAnchor,abscissaMidAnchor,abscissaRightAnchor,style,cs):
		result=[]
		#TODO additionalStyles für alle Labels und styles
		#
		# Convert displayed value (v)
		lowerValue = min(firstValue, secondValue)
		greaterValue = max(firstValue, secondValue)
		value = greaterValue - lowerValue
		if lowerValue == 0:
			v = greaterValue
		elif greaterValue == 0:
			v = lowerValue
		else:
			v=value
		if self.attributes.valueConversion=='int':
			if (v%1==0): v=int(v)
		if self.attributes.valueConversion=='round':
			v=int(round(v))
		if callable(self.attributes.valueConversion):
			v=self.attributes.valueConversion(v)
		#TODO mehr davon: länge des balken in baseunit, startseite des balken (left,right, top, down), endseite des balken
		#
		# make a dict with relevant data for this item (used for format string in labels and styles)
		startSide=''
		endSide=''
		if (self.attributes.abscissaDirection == 'right' or self.attributes.abscissaDirection == 'left'):
			if self.attributes.ordinateInversion:
				startSide='top'
				endSide='bottom'
			else:
				startSide='bottom'
				endSide='top'
		else:
			if self.attributes.ordinateInversion:
				startSide='left'
				endSide='right'
			else:
				startSide='right'
				endSide='left'
		if value < 0:
			startSide, endSide = endSide, startSide
		itemDict={
			'v':v,
			'value':v,
			'p':percentage,
			'absPercentage':abs(percentage),
			'n':name,
			'name':name,
			'r':percentage/100,
			'absRatio':abs(percentage)/100,
			'max':maxValue,
			'min':minValue,
			'maxOrdinate':maxOrdinateValue,
			'minOrdinate':minOrdinateValue,
			#TODO: bug: maxValue-minValue can be zero
			#TODO: bug: may be the same as above: If a tables first row has a sum of zero, i get a divided by zero exception
			'totalRatio':(value-minValue) / (maxValue-minValue),
			'totalPercentage':100 * (value-minValue) / (maxValue-minValue),
			'totalRatioOrdinate':(value-minOrdinateValue) / (maxOrdinateValue-minOrdinateValue),
			'absMax':abs(maxValue),
			'absMin':abs(minValue),
			'absMaxOrdinate':abs(maxOrdinateValue),
			'absMinOrdinate':abs(minOrdinateValue),
			'absTotalRatio':abs(value) / max(abs(minValue),abs(maxValue)),
			'absTotalPercentage':100 * abs(value) / max(abs(minValue),abs(maxValue)),
			'totalRatioOrdinate':abs(value)-min(abs(minOrdinateValue),abs(maxOrdinateValue)) / (max(abs(minOrdinateValue),abs(maxOrdinateValue))-min(abs(minOrdinateValue),abs(maxOrdinateValue))),
			'startSide':startSide,
			'endSide':endSide,
		}
		#
		# draw the bar
		result.append(cs.drawRectangle((abscissaLeftAnchor,lowerValue),(abscissaRightAnchor,greaterValue),style+","+self.attributes.additionalBarStyle.format(**itemDict)))
		#
		# initialize the label texts
		olL=self.attributes.outerLowerLabelString.format(**dict(self.attributeDict.items()+itemDict.items()))
		ilL=self.attributes.innerLowerLabelString.format(**dict(self.attributeDict.items()+itemDict.items()))
		cL=self.attributes.centerLabelString.format(**dict(self.attributeDict.items()+itemDict.items()))
		iuL=self.attributes.innerUpperLabelString.format(**dict(self.attributeDict.items()+itemDict.items()))
		ouL=self.attributes.outerUpperLabelString.format(**dict(self.attributeDict.items()+itemDict.items()))
		# make lable flips that depend on bar length
		if abs(cs.realToTikzY(value)) < self.attributes.barLengthThreshold:
			if self.attributes.flipLowerAboveWhenThreshold: ilL,ouL = ouL,ilL
			if self.attributes.flipUpperAboveWhenThreshold: iuL,ouL = ouL,iuL
			if self.attributes.flipCenterAboveWhenThreshold: cL,ouL = ouL,cL
			if self.attributes.flipLowerBelowWhenThreshold: ilL,olL = olL,ilL
			if self.attributes.flipUpperBelowWhenThreshold: iuL,olL = olL,iuL
			if self.attributes.flipCenterBelowWhenThreshold: cL,olL = olL,cL
		# make label flips that depend on negative value
		if lowerValue < 0:
			if self.attributes.flipLowerLabelsWhenNegative:
				olL,ilL = ilL,olL
			if self.attributes.flipUpperLabelsWhenNegative:
				ouL,iuL = iuL,ouL
			if self.attributes.flipOuterLabelsWhenNegative:
				olL,ouL = ouL,olL
			if self.attributes.flipInnerLabelsWhenNegative:
				ilL,iuL = iuL,ilL
		#outer lower label
		if self.attributes.outerLowerLabelStyle != None and olL != "":
			style=(cs.getStyle('orientedLabelStyleBelow') if value > 0 else cs.getStyle('orientedLabelStyleAbove'))+","+cs.getStyle('outerLowerLabelStyle')
			style=style+','+self.attributes.additionalOuterLowerLabelStyle.format(**dict(self.attributeDict.items()+itemDict.items()))
			label=olL
			result.append("\\path {c} node[{style}] {{{label}}};".format(c=cs.tcoord((abscissaMidAnchor, lowerValue)),style=style,label=label))
		#inner lower label
		if self.attributes.innerLowerLabelStyle != None and ilL != "":
			style=(cs.getStyle('orientedLabelStyleAbove') if value > 0 else cs.getStyle('orientedLabelStyleBelow'))+","+cs.getStyle('innerLowerLabelStyle')
			style=style+','+self.attributes.additionalInnerLowerLabelStyle.format(**dict(self.attributeDict.items()+itemDict.items()))
			label=ilL
			result.append("\\path {c} node[{style}] {{{label}}};".format(c=cs.tcoord((abscissaMidAnchor,lowerValue)),style=style,label=label))
		#center
		if self.attributes.centerLabelStyle != None and cL != "":
			style=cs.getStyle('orientedLabelStyleCenter')+","+cs.getStyle('centerLabelStyle')
			style=style+','+self.attributes.additionalCenterLabelStyle.format(**dict(self.attributeDict.items()+itemDict.items()))
			label=cL
			result.append("\\path {c} node[{style}] {{{label}}};".format(c=cs.tcoord((abscissaMidAnchor,lowerValue+value/2)),style=style,label=label))
		#inner upper label
		if self.attributes.innerUpperLabelStyle != None and iuL != "":
			style=(cs.getStyle('orientedLabelStyleBelow') if value > 0 else cs.getStyle('orientedLabelStyleAbove'))+","+cs.getStyle('innerUpperLabelStyle')
			style=style+','+self.attributes.additionalInnerUpperLabelStyle.format(**dict(self.attributeDict.items()+itemDict.items()))
			label=iuL
			result.append("\\path {c} node[{style}] {{{label}}};".format(c=cs.tcoord((abscissaMidAnchor,greaterValue)),style=style,label=label))
		#outer upper label
		if self.attributes.outerUpperLabelStyle != None and ouL != "":
			style=(cs.getStyle('orientedLabelStyleAbove') if value > 0 else cs.getStyle('orientedLabelStyleBelow'))+","+cs.getStyle('outerUpperLabelStyle')
			style=style+','+self.attributes.additionalOuterUpperLabelStyle.format(**dict(self.attributeDict.items()+itemDict.items()))
			label=ouL
			result.append("\\path {c} node[{style}] {{{label}}};".format(c=cs.tcoord((abscissaMidAnchor,greaterValue)),style=style,label=label))
		#TODO: low : Schriftdefinitionen aus den tikz-styles in separate defs auslagern
		#
		return '\n'.join(result)
	
	def getCsAndMinAndMax(self,realMinValue,realMaxValue,width):
		if self.attributes.ordinateLength==None and self.attributes.ordinateTickDelta==None:
			self.attributes.extend(ordinateLength=self.attributes.fallbackOrdinateLength)
		if self.attributes.ordinateMin == None:
			minValue=decimalCeil(min(realMinValue,0))
			self.attributes.extend(ordinateMin=minValue)
		if self.attributes.ordinateMax == None:
			maxValue=decimalCeil(max(realMaxValue,0))
			self.attributes.extend(ordinateMax=maxValue)
		if self.attributes.abscissaLength==None:			
			self.attributes.extend(abscissaLength=width)
		self.attributes.extend(abscissaDomain=self.attributes.abscissaLength)
		self.attributes.extend(abscissaMin=-self.attributes.startMargin)
		self.attributes.extend(ordinateValue=-self.attributes.startMargin)
		#	
		self.attributes.extend(abscissaTickDelta=0)
		#
		cs = TikZCoordinateSystem(**self.attributes.getDict())
		return cs,minValue,maxValue
	
	def render(self,table):
		rowCount = table.getRowCount()
		colCount = table.getColCount()
		# initialize
		total = []
		realMinValue=float(table.getCell(0,0))
		realMaxValue=float(table.getCell(0,0))
		values = []
		for j in range(0,rowCount):
			values.append([])
			total.append(0.0)
			for i in range(0,colCount):
				try:
					cell=table.getCell(i,j)
					try:
						f = float(cell)
					except:
						f=0
					values[j].append(f)
					total[j] += f
					realMinValue = min(f,realMinValue)
					realMaxValue = max(f,realMaxValue)
				except ValueError, e:
					raise ziktRenderException( "Minimum one cell does not contain a valid number. (%s)" % str(e))
#		n = len(values)
#		width=self.attributes.startMargin+self.attributes.endMargin+n*self.attributes.barWidth+(n-1)*self.attributes.midMargin
		#print rowCount," ",colCount
		groupWidth = (
			colCount * self.attributes.barWidth +
			(colCount - 1) * self.attributes.barMargin
		)
		width = (
			self.attributes.startMargin +
			self.attributes.endMargin +
			(rowCount - 1) * self.attributes.midMargin +
			(rowCount) * groupWidth
		)
			
		cs,minValue,maxValue = self.getCsAndMinAndMax(realMinValue,realMaxValue,width)
			
		# building output
		result = []
		
		result.append(cs.getTikZ())
		
		#draw the n × m bars
		for j in range(0,rowCount):
			maxInRow=values[j][0]
			minInRow=values[j][0]
			for i in range(0,colCount):
				if values[j][i] > maxInRow: maxInRow=values[j][i]
				if values[j][i] < minInRow: minInRow=values[j][i]
				leftAnchor = (
					j * (groupWidth + self.attributes.midMargin) +
					i * (self.attributes.barWidth + self.attributes.barMargin)
				)
				result.append(self.drawBar(**{
					'index':i,
					'firstValue':0, 
					'secondValue':values[j][i],
					'percentage':100*values[j][i]/total[j] if total[j] != 0 else 0,
					'maxValue':realMaxValue,
					'minValue':realMinValue,
					'maxOrdinateValue':maxValue,
					'minOrdinateValue':minValue,
					'name':table.getColHeader(i),
					'abscissaLeftAnchor': leftAnchor,
					'abscissaMidAnchor': (0.5)*self.attributes.barWidth + leftAnchor,
					'abscissaRightAnchor': self.attributes.barWidth + leftAnchor,
					'style':(self.attributes.barStyle if self.attributes.barStyleMode == 'single' else self.attributes.barStyles[i % len(self.attributes.barStyles)]),
					'cs':cs
				}))
			leftAnchor= j * (groupWidth + self.attributes.midMargin)
			midAnchor = leftAnchor + groupWidth/2
			#TODO: low: das wurde auskommentiert weil es nicht genutzt wurde; ich weiß aber nicht warum das da steht...
			#rightAnchor = leftAnchor + groupWidth
			groupDict = {
				'n':table.getRowHeader(j),
				'name':table.getRowHeader(j),
				'total':total[j],
			}
			if self.attributes.groupUpperLabelString != "" and self.attributes.groupUpperLabelStyle != None:
				style = cs.getStyle('groupUpperLabelStyle') + ',' + cs.getStyle('orientedLabelStyleAbove') + ',' + self.attributes.additionalGroupUpperLabelStyle.format(**groupDict)
				label = self.attributes.groupUpperLabelString.format(**groupDict)
				result.append("\\path {c} node [{style}] {{{label}}};".format(c=cs.tcoord((midAnchor,max(maxInRow,0))),style=style,label=label))
			if self.attributes.groupLowerLabelString != "" and self.attributes.groupLowerLabelStyle != None:
				style = cs.getStyle('groupLowerLabelStyle') + ',' + cs.getStyle('orientedLabelStyleBelow') + ',' + self.attributes.additionalGroupLowerLabelStyle.format(**groupDict)
				label = self.attributes.groupLowerLabelString.format(**groupDict)
				result.append("\\path {c} node [{style}] {{{label}}};".format(c=cs.tcoord((midAnchor,min(minInRow,0))),style=style,label=label))
		return "\n".join(result)

#t = FloatTable()
#t.addRow([0.5,1,1])
#t.addRow([0.5,1,1])
#b = BarChart()
#b.render(t)



# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

import unittest
from zikt.tables_tests import getBasicTable
from zikt.tables import FloatTable
from zikt.stackedbarchart import StackedBarChart
from zikt.cs import TikZCoordinateSystem

class TableTests(unittest.TestCase):
    
    def test_noFailureOnAbscissaLengthSpecified(self):
        table = FloatTable(getBasicTable())
        chart = StackedBarChart(
#                alongsideOrdinateLabelString = 'work left',
#                outerOrdinateLabelString = 'SP',
#                lowestLabelString = '{colHeader}',
                abscissaLength = 10,
#                additionalLowestLabelStyle = 'font=\\scriptsize, rotate=90, anchor=east'
        )
        _ = chart.render(table)


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from __future__ import division
from zikt.cschart import MultipleValueCoordinateChart

#TODO:high: test painters with tables containing significant null value patterns

def getLabelNode(coord, name,attributeDict,renderer,orientation=None):
	style = renderer.attributeDict[name+"Style"]
	string = renderer.attributeDict[name+"String"]
	if style != None and string != None and string != "":
		string = string.format(**attributeDict)
		style = style.format(**attributeDict)
		if orientation  == None:
			orientationStyle = ""
		else:
			orientationStyle = renderer.getStyle('orientedLabelStyle' + orientation) + ","
		return "\\path ({c}) node[{orientationStyle}{style}] {{{label}}};".format(
			c=coord,
			orientationStyle=orientationStyle,
			style=style,
			label=string
		)
	else:
		return None
	##
##
		
class PlotPainter():
	@staticmethod
	def paint(renderer,table,labelTable,rowIndex):
		lineResult = []
		markResult = []
		labelResult = []
		lastCoord=None
		for colIndex in range(table.getColCount()):
			if table.getCell(colIndex,rowIndex) == None:
				continue
			coord = MultipleValueCoordinateChart.getValueCoordinateName(colIndex,rowIndex)
			formatDict = renderer.getCellDict(colIndex,rowIndex,table,labelTable)
			formatDict.update({
				'mark' : renderer.getRowAttribute(rowIndex,'plotMarks'),
			})
			# the label above
			labelAbove = getLabelNode(coord, "valuePointLabelAbove", formatDict, renderer, orientation = 'Above')
			if labelAbove != None:
				labelResult.append(labelAbove)
			# the label below
			labelBelow = getLabelNode(coord, "valuePointLabelBelow", formatDict, renderer, orientation = 'Below')
			if labelBelow != None:
				labelResult.append(labelBelow)
			# the plot marks
			if renderer.attributes.plotMarkStyle != None:
				markStyle = (renderer.attributes.plotMarkStyle + ", " + renderer.attributes.additionalPlotMarkStyle).format(**dict(formatDict.items()))
				markResult.append("\\path[{style}] plot coordinates {{({c})}};".format(c=coord, style=markStyle))
			# the plot line
			if renderer.attributes.lineStyle != None:
				lineStyle = (renderer.attributes.lineStyle + ", " + renderer.attributes.additionalLineStyle).format(**dict(formatDict.items()))
				if lastCoord !=None:
					lineResult.append("\path[{style}] ({lastC}) -- ({c});".format(lastC=lastCoord, c=coord, style=lineStyle))
			# for the next iteration... :)			
			lastCoord = coord
			##
		return '\n'.join(['\n'.join(lineResult),'\n'.join(markResult),'\n'.join(labelResult)])
		##

class BarPainter:
	@staticmethod
	def paint(renderer,table,labelTable,rowIndex):
		if rowIndex == 0:
			return ""
		barResult = []
		labelResult = []
		for colIndex in range(table.getColCount()):
			if table.getCell(colIndex,rowIndex) == None:
				continue
			##
			coord = MultipleValueCoordinateChart.getValueCoordinateName(colIndex,rowIndex)
			lowerCoord = MultipleValueCoordinateChart.getValueCoordinateName(colIndex,rowIndex-1)
			formatDict = renderer.getCellDict(colIndex,rowIndex,table,labelTable,designOffset=-1)
			# the bar
			barStyle = (renderer.attributes.barStyle + "," + renderer.attributes.additionalBarStyle).format(**formatDict)
			offset = (renderer.attributes.abscissaTickDelta * renderer.attributes.barWidthRatio) / 2
			v1 = renderer.tcoord(renderer.transformTikZVector(-offset,0))
			v2 = renderer.tcoord(renderer.transformTikZVector(offset,0))
			
			barResult.append(
				"\\path[coordinate] ({c2}) ++{v2} coordinate(t);\\path[{style}] ({c1}) ++{v1} rectangle (t);".format(
					style=barStyle,
					c1=coord,
					c2=lowerCoord,
					v1=v1,
					v2=v2,
					offset=offset,
					unit=renderer.attributes.baseunit,
				)
			)
		return '\n'.join(['\n'.join(barResult),'\n'.join(labelResult)])


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

import sys
import csv

class CSV:
	def __init__(self,delimiter=None,quotechar=None):
		self.delimiter = delimiter
		self.quotechar = quotechar
			
	def render(self,table):
		array = table.getTableArray()
		writer = csv.writer(sys.stdout)
		for item in array:
			writer.writerow(item)
		


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''
import unicodecsv as csv
import math
from zikt.helper import ziktTableException
from operator import itemgetter




class Table:
	"""
	The central class that models a single table. All input data is turned into a table object and all output/renderer classes take their information from an C{Table}-Object.
	
	A table consists of a table body that is an two dimensional array. The outer dimension contains the rows, the inner dimension contains the cells of the concerning row. Furthermore, the table can optionally own row headers and/or column headers.
	"""
		
	def __init__(self):
		self.rowHeader=[]
		self.colHeader=[]
		self.rows=[]
		self.hasRowHeader=True
		self.hasColHeader=True
	
	#TODO: refactor: adding a row should be separated into a two-param function: the row label and the content cell array;... 
	#this could be the function declaration; addRow(self, rowLabel="", row) where row is the appendable, one-dimensional cell collection
	def addRow(self,row):
		if self.colHeader == [] and self.hasColHeader:
			self.colHeader = row if not self.hasRowHeader else row[1:]
		else:
			if self.hasRowHeader:
				self.rowHeader.append(row[0])
				self.rows.append(row[1:])
			else:
				self.rows.append(row)
				
	def insertRow(self,index,rowData,rowHeader=None):
		self.rows.insert(index, rowData)
		if self.hasRowHeader:
			self.rowHeader.insert(index, rowHeader)

	def __iter__(self):
		return iter(self.rows)

	def getMinRowLength(self):
		if len(self.rows) == 0:
			return 0
		else:
			result = len(self.rows[0])
			for row in self.rows:
				if len(row) < result:
					result = len(row)
			return result

	def getMaxRowLength(self):
		if len(self.rows) == 0:
			return 0
		else:
			result = len(self.rows[0])
			for row in self.rows:
				if len(row) > result:
					result = len(row)
			return result
			
	def getColHeader(self,index):
		if not self.hasColHeader: return ""
		return self.colHeader[index]
	
	def getColHeaders(self):
		if not self.hasColHeader: return ""
		return self.colHeader
		
	def getRowHeader(self, index):
		if not self.hasRowHeader: return ""
		return self.rowHeader[index]
	
	def getRowHeaders(self):
		if not self.hasRowHeader: return ""
		return self.rowHeader
		
	def getRowCount(self):
		return len(self.rows)
		
	def getColCount(self):
		return self.getMaxRowLength()
		
	def getCell(self,colIndex,rowIndex):
		if rowIndex > len(self.rows)-1:
			raise ziktTableException("row index out of bounds (asked for index %i but have only %i + 1 rows)" %(rowIndex,  len(self.rows)-1))
		if colIndex > self.getColCount()-1:
			raise ziktTableException("col index out of bounds (asked for index %i but have only %i + 1 columns)" %(colIndex,  self.getColCount()-1))
		r = self.rows[rowIndex]
		if colIndex > len(r)-1:
			return None
		return r[colIndex]
	
	def setCell(self, colIndex, rowIndex, value):
		try:
			row = self.rows[rowIndex]
		except IndexError:
			raise ziktTableException("Row index out of bounds.")
		if colIndex > self.getColCount() - 1:
			raise ziktTableException("Column index out of bounds.")
		fill = colIndex - (len(row)-1)
		if fill > 0:
			appendix = [None for x in range(fill-1)]
			appendix.extend([value])
			row.extend(appendix)
		else:
			row[colIndex] = value
		
		
	def swap(self):
		self.colHeader,self.rowHeader = self.rowHeader,self.colHeader
		self.hasRowHeader, self.hasColHeader = self.hasColHeader, self.hasRowHeader
		newTable=[]
		for i in range(self.getColCount()):
			row=[]
			for j in range(self.getRowCount()):
				row.append(self.getCell(i,j))
			newTable.append(row)
		self.rows = newTable
		
	#DODO: Refactor getRow() so it does not return the header field as first element (if the header field exists, of cause)
	def getRow(self,index):
		return self.rows[index]
	
	def getCol(self, index):
		result=[]
		for i in range(self.getRowCount()):
			c = self.getCell(index, i)
			if c != None:
				result.append(c)
		return result
		
	def getTableArray(self):
		result=[]
		if self.hasColHeader:
			if self.hasRowHeader:
				result.append(['']+self.colHeader)
			else:
				result.append(self.colHeader)
		for i in range(self.getRowCount()):
			if self.hasRowHeader:
				result.append([self.getRowHeader(i)]+self.getRow(i))
			else:
				result.append(self.getRow(i))
		return result

	def unfoldRow(self,rowIndex, fillValue=None,line_delimiter=',',tuple_delimiter=':'):
		row = self.rows.pop(rowIndex)
		if self.hasRowHeader:
			self.rowHeader.pop(rowIndex)
		# extract data
		all_keys = []
		columns_maps = []
		for cell in row:
			column_map = {}
			if cell != None:
				lines = cell.split(line_delimiter)
				for line in lines:
					if line != "":
						value_tuple = line.split(tuple_delimiter)
						if len(value_tuple) != 2:
							raise ziktTableException("Cell content to unfold has a line value that can not be splitted into a key and a value. (%s)"%line)
						key = value_tuple[0]
						value = value_tuple[1]
						if key in column_map:
							raise ziktTableException("Cell content to unfold has an ambiguous key. (%s)"%key)
						column_map[key] = value
						if key not in all_keys:
							all_keys.append(key)
						##
					##
				##
			columns_maps.append(column_map)
		# insert new rows
		for key in reversed(all_keys):
			new_row = []
			if self.hasRowHeader:
				self.rowHeader.insert(rowIndex, key)
			for column_map in columns_maps:
				if key in column_map:
					value = column_map[key]
				else:
					value = fillValue
				new_row.append(value)
			self.rows.insert(rowIndex,new_row)

		

class StringTable(Table):
	def __init__(self,infile=None,delimiter=',', quotechar='"'):
		Table.__init__(self)
		
		if infile!=None:
			self.initByCSV(infile,delimiter=delimiter,quotechar=quotechar)
	
	def initByCSV(self,infile,delimiter=',', quotechar='"'):
		csvReader = csv.reader(infile, delimiter=delimiter, quotechar=quotechar)
		for row in csvReader:
			self.addRow(row)



class FloatTable(Table):
	def __init__(self, table=None, hasRowHeader=None, hasColHeader=None):
		Table.__init__(self)
		if table != None:
			if isinstance(table,Table): 
				self.colHeader = table.colHeader
				self.hasColHeader = table.hasColHeader
				self.hasRowHeader = table.hasRowHeader
				for rowNumber in range(table.getRowCount()):
					newRow=[]
					if table.hasRowHeader:
						newRow.append(table.getRowHeader(rowNumber))
					row = table.getRow(rowNumber)
					for element in row:
						try:
							if element == None:
								newRow.append(None)
							else:
								newRow.append(float(element))
						except ValueError:
							raise ziktTableException("Cell content can not be converted to float value.")
					self.addRow(newRow)
			else:
				if hasColHeader == None:
					self.hasColHeader = True
				else:
					self.hasColHeader = hasColHeader
				if hasRowHeader == None:  
					self.hasRowHeader = False
				else:
					self.hasRowHeader = hasRowHeader
				for row in table:
					self.addRow(row)
				
				
	def setCell(self, colIndex, rowIndex, value):
		if value != None:
			try:
				value = float(value)
			except:
				raise ziktTableException("The value to set is not a float.")
		Table.setCell(self, colIndex, rowIndex, value)
				
	def getMinMax(self):
		rowCount = self.getRowCount()
		colCount = self.getColCount()
		if colCount < 1:
			raise ziktTableException("no data columns in table")
		if rowCount < 1:
			raise ziktTableException("no data rows in table")
		minValue = self.getCell(0, 0)
		maxValue = self.getCell(0, 0)
		for i in range (0, colCount):
			for j in range (0, rowCount):
				v = self.getCell(i, j)
				#TODO: bug: min/max-Berechnungen berücksichtigen die Nones nicht
				#TODO: hack: lokaler work-around für obigen bug
				if v == None:
					break
				if v < minValue:
					minValue = v
				if v > maxValue:
					maxValue = v
		return minValue, maxValue
	
	def getMinMaxOfCol(self, colIndex):
		if self.getRowCount() < 1:
			raise ziktTableException("No data rows in table; No Min/Max calculations possible")
		col = self.getCol(colIndex)
		col.sort()
		return col[0],  col[len(col)-1]
		
	def getMinMaxIndicesOfCol(self, colIndex):
		indexValuePairs=[]
		col=self.getCol(colIndex)
		for i in range(len(col)):
			indexValuePairs.append((i, col[i]))
		sortedPairs = sorted(indexValuePairs,  key=itemgetter(1))
		return sortedPairs[0][0],  sortedPairs[len(sortedPairs)-1][0]
	
	def getColSum(self,columnIndex):
		return FloatTable.sumList(self.getCol(columnIndex))
	
	def getColRange(self,columnIndex):
		minV, maxV = self.getMinMaxOfCol(columnIndex)
		return abs(minV-maxV)
	
	def getMinMaxColSum(self):
		if self.getColCount() < 1:
			return None
		firstCol = self.getCol(0)
		firstColSum = FloatTable.sumList(firstCol)
		minV = firstColSum
		maxV = firstColSum
		for i in range(self.getColCount()):
			colsum = self.getColSum(i)
			minV = min(minV,colsum)
			maxV = max(maxV,colsum)
		return minV, maxV
	
	def allPositive(self):
		for i in range(self.getColCount()):
			col = self.getCol(i)
			for c in col:
				if c < 0:
					return False
				##
			##
		return True
	
	def allPositiveExceptFirstRow(self):
		rowCount = self.getRowCount()
		i = 1
		while i < rowCount:
			for c in self.getRow(i):
				if c < 0:
					return False
				##
			i += 1
		return True
	
	def allPositiveInColumn(self,columnIndex,exceptFirst=False):
		column = self.getCol(columnIndex)
		if exceptFirst:
			column = column[1:]
		for cell in column:
			if cell < 0:
				return False
		return True
	
	def getStackedColumn(self,columnIndex):
		if not self.allPositiveInColumn(columnIndex,exceptFirst=True):
			raise ziktTableException("To get a stacked column, all values in the column must be positive.")
		result = []
		column = self.getCol(columnIndex)
		stacksum = 0
		for cell in column:
			stacksum += cell
			result.append(stacksum)
		return result
			
	
	def getStacked(self,onTopOfZero):
		if (onTopOfZero):
			if not self.allPositive():
				raise ziktTableException("When making a stacked table on top of zero, all values in table must be positive.")
			##
		else:
			if not self.allPositiveExceptFirstRow():
				raise ziktTableException("When making a stacked table on top of the first row, all values in the second and following rows must be positive.")
			##
		result = FloatTable(table = self)
		if onTopOfZero:
			firstRow = [0 for x in range(self.getColCount())]
			result.insertRow(0, firstRow, "")
		for columnIndex in range(result.getColCount()):
			stackedColumn = result.getStackedColumn(columnIndex)
			for cellIndex in range(result.getRowCount()):
				result.setCell(columnIndex, cellIndex, stackedColumn[cellIndex] if cellIndex < len(stackedColumn) else None)
			##
		return result
			
		
	@staticmethod
	def sumList(floatlist):
		listsum = 0
		for i in floatlist:
			listsum += i
		return listsum
		



# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from zikt.cschart import MultipleValueCoordinateChart
from zikt.helper import Attributes
from zikt.painter import PlotPainter
from zikt.tables_tests import getBasicTable
from zikt.tables import FloatTable

class PlotChart(MultipleValueCoordinateChart):
	base = Attributes(
		parent=MultipleValueCoordinateChart.base,
		painter=[PlotPainter],
		#
		valuePointLabelAboveString=u"{value}",
		valuePointLabelAboveStyle=u"",
		valuePointLabelBelowString=u"",
		valuePointLabelBelowStyle=u"",
		#
		plotMarks = ["square*"],
		plotMarkStyle = "ziktPlotChartMark",
		additionalPlotMarkStyle = u"mark={mark},mark options={{color={outlineColor}}}",
		#
		lineStyle = u"ziktPlotChartLine",
		additionalLineStyle = u"color={drawColor}"
	)
	presettings =  {
		'base': base,
	}
	
	def __init__(self,preset="base", printComments=True, **kwargs):
		MultipleValueCoordinateChart.__init__(self)
		self.attributes = PlotChart.presettings[preset]
		self.attributes.extend(**kwargs)
		self.printComments = printComments
		self.attributeDict = self.attributes.getDict()
		

if __name__ == "__main__":
	c =  PlotChart(ordinateLength=6,stackmode = True)
	print c.render(table = FloatTable(getBasicTable()))


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from __future__ import division
from zikt.helper import ziktRenderException, Attributes
#import math
from zikt.cs import TikZCoordinateSystem
from legend import Legend
#import sys
import copy


class MultipleValueCoordinateChart(TikZCoordinateSystem):
	"""
	A very basic renderer that take n lines from the table as input and defines TikZ coordinates aliases at the coresponding coordinates.
	"""
	base = Attributes(
		parent=TikZCoordinateSystem.empty_preset,
		#
		relevantRowCount = 2,  # how many rows from the input table are used to print a value coordinate
		stackmode = False, # if true, the values’ coordinates are not set directly by the value but “stacked” by adding the value to all former values 
		ordinateLength = 5,  # default length of the ordinate
		alwaysIncludeZero = False, # if true, ordinate domain is enlarged to include zero if it does not contain zero naturally
		#TODO: low : add additional upper and lower space to this AND the BarChart (maybe common super class?); the additional space could be specifed as percentage of the ordinate domain
		#additionalUpperOrdinateDomain = None, # percentage of the ordinate domain that get added to the upper end of the ordinate to get some graphical space above the highest coordinate
		#additionalLowerOrdinateDomain = None, # percentage of the ordinate domain that get added to the lower end of the ordinate to get some graphical space above the highest coordinate
		#
		# Margins
		#
		startMargin=0.7,  # margin before the first column
		endMargin=0.7,  # margin after the first column
		columnMargin=1,  # margin between two columns
		#
		# TikZ classes for colors
		#
		#TODO: low (until working on graphs that are not derived from cschart): move this "row classes" to the most upper renderer class
		drawColors = ["colorElementOneDraw","colorElementTwoDraw","colorElementThreeDraw","colorElementFourDraw","colorElementFiveDraw","colorElementSixDraw","colorElementSevenDraw","colorElementEightDraw","colorElementNineDraw","colorElementTenDraw","colorElementElevenDraw","colorElementTwelve"],
		fillColors = ["colorElementOneFill","colorElementTwoFill","colorElementThreeFill","colorElementFourFill","colorElementFiveFill","colorElementSixFill","colorElementSevenFill","colorElementEightFill","colorElementNineFill","colorElementTenFill","colorElementElevenFill","colorElementTwelveFill"], 
		outlineColors = ["colorElementOneOutline","colorElementTwoOutline","colorElementThreeOutline","colorElementFourOutline","colorElementFiveOutline","colorElementSixOutline","colorElementSevenOutline","colorElementEightOutline","colorElementNineOutline","colorElementTenOutline","colorElementElevenOutline","colorElementTwelveOutline"], 
		#
		# TikZ strings
		#
		columnTikZString = None, 
		valueTikZString = None, 
		#
		# Labels (styles, additional styles and strings)
		#
		maximumLabelStyle=u'',
		minimumLabelStyle=u'',
		highestLabelStyle=u'',
		lowestLabelStyle=u'',
		#
		additionalMaximumLabelStyle=u'',
		additionalMinimumLabelStyle=u'',
		additionalHighestLabelStyle=u'',
		additionalLowestLabelStyle=u'',
		#
		maximumLabelString=u'',
		minimumLabelString=u'',
		highestLabelString=u'',
		lowestLabelString=u'',
		#
		painter=None,
		legend=False,
		legendFont=None,
	)
	presettings =  {
		'base': base,
	}
	
	def __init__(self,preset="base", printComments=True, **kwargs):
		TikZCoordinateSystem.__init__(self)
		self.attributes = MultipleValueCoordinateChart.presettings[preset].copy()
		self.attributes.extend(**kwargs)
		self.printComments = printComments
		self.attributeDict = self.attributes.getDict()
		# ToDo: make the values for the value columns flexible by using a table row; that way, this chart can be used even for specific numerical indices of the value columns; …
		# if the array is None, the values will be automatically calculates with omni distances for the value columns; if the values come from the input table, the columnMargin attribute
		# will not be used for the rendering.
		self.valueColumnsValues = None
		
	def getRowAttribute(self, rowIndex, attributeName):
		attribute = self.attributeDict[attributeName]
		if not isinstance(attribute, list):
			return attribute
		else:
			return attribute[rowIndex % len(attribute)]
		
	@staticmethod
	def getValueCoordinateName(colIndex, rowIndex):
		return "v-%i-%i"%(colIndex, rowIndex)
	
	def getValueColumn(self, colIndex, table,  minValue,  maxValue, labelTable = None):
		if labelTable == None:
			labelTable = table
		result=[]
		col = table.getCol(colIndex)
		labelCol = labelTable.getCol(colIndex)
		# calculate sum (total)
		total = 0
		for cell in col:
			total += cell
		#
		# define the value coordinates
		for i in range(len(col)):
			coordinateName = MultipleValueCoordinateChart.getValueCoordinateName(colIndex, i)
			coordinate = self.tcoord((colIndex, col[i]))
			result.append( "\\coordinate (%s) at %s;"%(coordinateName, coordinate))
		#
		# set the coordinates on the outer lines and the abscissa
		maxCoordinate = self.tcoord((colIndex, maxValue))
		minCoordinate = self.tcoord((colIndex, minValue))
		result.append( "\\coordinate (%s) at %s;"%("min-"+str(colIndex), minCoordinate))
		result.append( "\\coordinate (%s) at %s;"%("max-"+str(colIndex), maxCoordinate))
		abscissaCoordinate = None
		if self.abscissaDefined:
			abscissaCoordinate = self.tcoord((colIndex, self.attributes.abscissaValue))
			result.append( "\\coordinate (%s) at %s;"%("abscissa-"+str(colIndex), abscissaCoordinate))
		#
		minColValue, maxColValue = table.getMinMaxOfCol(colIndex)
		minColValueIndex,  maxColValueIndex = table.getMinMaxIndicesOfCol(colIndex)
		lowestValueCoordinate = MultipleValueCoordinateChart.getValueCoordinateName(colIndex, minColValueIndex)
		highestValueCoordinate = MultipleValueCoordinateChart.getValueCoordinateName(colIndex, maxColValueIndex)
		minimumCoordinate = "min-"+str(colIndex)
		maximumCoordinate = "max-"+str(colIndex)
		#
		columnDict={
			'lowestValueCoordinate' : lowestValueCoordinate, 
			'highestValueCoordinate' : highestValueCoordinate,
			'minimumCoordinate' : minimumCoordinate,
			'maximumCoordinate' : maximumCoordinate,
			'abscissaCoordinate' : "abscissa-"+str(colIndex),
			'colHeader' : table.getColHeader(colIndex), 
			'minValue' : minColValue, 
			'maxValue' : maxColValue,
			'total' : total,
		}
		# 
		# column labels
		# maximum label
		if (self.attributes.maximumLabelString != None) and (self.attributes.maximumLabelString != ""):
			style = self.getStyle("orientedLabelStyleAbove")
			if (self.attributes.maximumLabelStyle != None) and (self.attributes.maximumLabelStyle != ""):
				style +=  "," + self.attributes.maximumLabelStyle
			if (self.attributes.additionalMaximumLabelStyle != None) and (self.attributes.additionalMaximumLabelStyle != ""):
				style +=  "," + self.attributes.additionalMaximumLabelStyle.format(**dict(self.attributeDict.items()+columnDict.items()))
			string = self.attributes.maximumLabelString.format(**dict(self.attributeDict.items()+columnDict.items()))
			result.append("\\path ({c}) node[{style}] {{{label}}};".format(c=maximumCoordinate, style=style, label=string))
		# minimum label
		if (self.attributes.minimumLabelString != None) and (self.attributes.minimumLabelString != ""):
			style = self.getStyle("orientedLabelStyleBelow")
			if (self.attributes.minimumLabelStyle != None) and (self.attributes.minimumLabelStyle != ""):
				style +=  "," + self.attributes.minimumLabelStyle
			if (self.attributes.additionalMinimumLabelStyle != None) and (self.attributes.additionalMinimumLabelStyle != ""):
				style +=  "," + self.attributes.additionalMinimumLabelStyle.format(**dict(self.attributeDict.items()+columnDict.items()))
			string = self.attributes.minimumLabelString.format(**dict(self.attributeDict.items()+columnDict.items()))
			result.append("\\path ({c}) node[{style}] {{{label}}};".format(c=minimumCoordinate, style=style, label=string))
		# highest label
		if (self.attributes.highestLabelString != None) and (self.attributes.highestLabelString != ""):
			style = self.getStyle("orientedLabelStyleAbove")
			if (self.attributes.highestLabelStyle != None) and (self.attributes.highestLabelStyle != ""):
				style +=  "," + self.attributes.highestLabelStyle
			if (self.attributes.additionalHighestLabelStyle != None) and (self.attributes.additionalHighestLabelStyle != ""):
				style +=  "," + self.attributes.additionalHighestLabelStyle.format(**dict(self.attributeDict.items()+columnDict.items()))
			string = self.attributes.highestLabelString.format(**dict(self.attributeDict.items()+columnDict.items()))
			result.append("\\path ({c}) node[{style}] {{{label}}};".format(c=highestValueCoordinate, style=style, label=string))
		# lowest label
		if (self.attributes.lowestLabelString != None) and (self.attributes.lowestLabelString != ""):
			style = self.getStyle("orientedLabelStyleBelow")
			if (self.attributes.lowestLabelStyle != None) and (self.attributes.lowestLabelStyle != ""):
				style +=  "," + self.attributes.lowestLabelStyle
			if (self.attributes.additionalLowestLabelStyle != None) and (self.attributes.additionalLowestLabelStyle != ""):
				style +=  "," + self.attributes.additionalLowestLabelStyle.format(**dict(self.attributeDict.items()+columnDict.items()))
			string = self.attributes.lowestLabelString.format(**dict(self.attributeDict.items()+columnDict.items()))
			result.append("\\path ({c}) node[{style}] {{{label}}};".format(c=lowestValueCoordinate, style=style, label=string))
		#
		# write the column TikZ string
		if (self.attributes.columnTikZString != None) and (self.attributes.columnTikZString != ""):
			columnTikZString=self.attributes.columnTikZString.format(**dict(self.attributes.getDict().items()+columnDict.items()))
			result.append(columnTikZString)
		#
		# write the value TikZ strings
		if (self.attributes.valueTikZString != None) and (self.attributes.valueTikZString != ""):
			for i in range(len(col)):
				valueDict = {
					'index' : i, 
					'value' : labelCol[i],
					'paintValue' : col[i], 
					'coordinate' : MultipleValueCoordinateChart.getValueCoordinateName(colIndex, i), 
					'fillColor' : self.attributes.fillColors[i % len(self.attributes.fillColors)], 
					'outlineColor' : self.attributes.outlineColors[i % len(self.attributes.outlineColors)], 
				}
				valueTikZString=self.attributes.valueTikZString.format(**dict(self.attributes.getDict().items()+columnDict.items()+valueDict.items()))
				result.append(valueTikZString)
		#
		return "\n".join(result)
	
	def render(self,table):
		labelTable = table
		originalTable = copy.deepcopy(table)
		if self.attributes.stackmode:
			if self.attributes.stackmode == "onTopOfZero":
				onTopOfZero = True
			elif self.attributes.stackmode == "onTopOfFirstRow":
				onTopOfZero = False
			else:
				raise ziktRenderException("unknown stack mode %s"%(self.attributes.stackmode))
			table = table.getStacked(onTopOfZero = onTopOfZero)	
			firstRow = [0 for x in range(table.getColCount())]
			labelTable.insertRow(0, firstRow, "")
		rowCount = table.getRowCount()
		colCount = table.getColCount()
		minValue,  maxValue = table.getMinMax()
		self.attributes.ordinateMin = minValue
		self.attributes.ordinateMax = maxValue
		#
		# if wanted, extend ordinate domain to include zero if it’s not already in there
		#TODO: mach das in cs.initDimensions!!! 
		if self.attributes.alwaysIncludeZero:
			if minValue > 0:
				minValue = 0
			if maxValue < 0:
				maxValue = 0
		#floorValue = math.floor(minValue)
		#ceilValue = math.ceil(maxValue)
		#
		# some checks before we start
		if  self.attributes.relevantRowCount < 1:
			raise ziktRenderException( "number of relevant rows must be minimum one")
		if rowCount < self.attributes.relevantRowCount:
			raise ziktRenderException( "number of relevant rows is higher than the number of rows in the input table")
		if colCount < 1:
			raise ziktRenderException( "input table has no data columns")
		#
		# Calculate the abscissa attributes
		if self.valueColumnsValues == None:
			# The columns will be omni distant; we can use coordinates as we like and we do it that way:
			# The columns we be placed on whole numbers (0,1,2,…) so the domain of the abscissa is the number of columns plus the start and end margin in proportion.
			# The ordinate Value will be set to the negated proportional Value of the start margin.
			#
			# first, we calculate the length of the abscissa
			if self.attributes.abscissaLength == None:
				self.attributes.abscissaLength =  \
					self.attributes.startMargin + \
					self.attributes.endMargin + \
					self.attributes.columnMargin * colCount
			# calculate the margins in value space (column margin in value space is 1 per definition)
			valueStartMargin = self.attributes.startMargin / self.attributes.columnMargin 
			valueEndMargin = self.attributes.endMargin / self.attributes.columnMargin
			# set the abscissa attributes and the ordinate position
			self.attributes.abscissaDomain = valueStartMargin + valueEndMargin + colCount-1
			self.attributes.abscissaMin = -valueStartMargin
			self.attributes.ordinateValue = -valueStartMargin
			# set the tick delta
			self.attributes.abscissaTickDelta = 1
		else:
			raise ziktRenderException("specific valueColumnsValues not yet supported")
		
		# Output generation starts here!!
		# Get the coordinate system TikZ code
		result = [TikZCoordinateSystem.render(self, table)]
		
		minValue = self.attributes.ordinateMin
		maxValue = self.attributes.ordinateMax
		
		# Get the n value columns
		for n in range(colCount):
			result.append(self.getValueColumn(n, table,  minValue,  maxValue, labelTable))
			
		# Call the painters…
		if self.attributes.painter != None and len(self.attributes.painter) > 0:
			for rowIndex in range(table.getRowCount()):
				painter = self.attributes.painter[rowIndex % len(self.attributes.painter)]
				result.append(painter.paint(self,table,labelTable,rowIndex))
		
		# Check if a legend is wanted
		if self.attributes.legend:
			legend = Legend(originalTable,self.attributes)
			lowest = self.attributes.ordinateMin# + 1/self.attributes.ordinateRatio
			leftest = self.attributes.abscissaMin
			rightest = self.attributes.abscissaMax
			mid = leftest + (rightest - leftest) / 2
			
			anchor = self.tcoord((mid,lowest))
			
			result.append("\\coordinate(legendanchor) at %s;"%anchor)
			result.append("\\node[below=1cm of legendanchor, anchor=north,text width=10cm]{%s};"%legend.get_hacked_floating_legend())
			
			#TODO: next: mache getter ins CS um coordinaten für die „Ecken“ zu bekommen
			#TODO: dann: mache die Basiskonfiguration in eine map nach Legend
			#TODO: dann: extende die eigene Basiskonfiguration mit dieser Map
			#TDOD: dann: in die Map muss legendPosition mit [north south,east,west]
			#TODO: dann: vertical leitet sich daraus ab
			#TODO: dann: mache primitive ausgabe und refactor die anker u.s.w.
			#TODO: dann: füge abstände, etc in die konfig-map ein
			#TODO: dann: mach geile Ausgabe 
				
		
		# return the result :)
		return "\n".join(result)
	
	def getCellDict(self, colIndex, rowIndex, table, labelTable, designOffset = 0):
		minV, maxV = labelTable.getMinMaxOfCol(colIndex)
		minVpaint, maxVpaint = table.getMinMaxOfCol(colIndex)
		return {
			'value':labelTable.getCell(colIndex,rowIndex),
			'paintValue':table.getCell(colIndex,rowIndex),
			'row':rowIndex,
			'col':colIndex,
			'index':colIndex,
			'min':minV,
			'max':maxV,
			'paintMin':minVpaint,
			'paintMax':maxVpaint,
			'sum':table.getColSum(colIndex),
			'range':table.getColRange(colIndex),
			'fillColor' : self.getRowAttribute(rowIndex + designOffset,'fillColors'), 
			'outlineColor' : self.getRowAttribute(rowIndex + designOffset,'outlineColors'), 
			'drawColor' : self.getRowAttribute(rowIndex + designOffset,'drawColors'),
		}


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

import re

class TikZRenderer():
	# TODO: low : add comments to TikZ output everywhere
	def printComment(self, prefix,comment,outputlist):
		if self.printComments:
			s = re.sub(ur'^', prefix+u"% ", (u"-" * 80)+u"\n"+comment+u"\n"+(u"-" * 80), flags=re.M)
			outputlist.append(s)


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from zikt.tables import FloatTable
from zikt.barchart import BarChart

data = [
	['A','B','C'],
	[2,12,-3]
	]

table = FloatTable(data)

chart = BarChart()

chartCode = chart.render(table)

print "\\begin{tikzpicture}"
print chartCode
print "\\end{tikzpicture}"


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

from __future__ import division
from zikt.helper import ziktRenderException, Attributes, Coord
from math import ceil, log, pow, floor
import logging
from zikt.tikzrender import TikZRenderer
from zikt.tables import FloatTable

#TODO: vision: support non-linear axes (exponential, logarithmic,...)
#TODO: high: support additional style for (outer) axis labels; I had a problem with an non-breakable outer ordinate label because "text width" could not be specified
class TikZCoordinateSystem(TikZRenderer):
	empty_preset = Attributes(
			ordinateLength=None,
			ordinateMin=None,
			ordinateMax=None,
			ordinateDomain=None,
			ordinateTickDelta=None,  # distance in value space
			ordinateTickAnchorValue = 0,
			ordinateValue = 0,
			ordinateRatio=None, # domain * ratio = length
			ordinateForcedDisplayMin = None,
			ordinateForcedDisplayMax = None,
			#
			abscissaLength=None,
			abscissaMin=None,
			abscissaMax=None,
			abscissaDomain=None,
			abscissaTickDelta=None,
			abscissaTickAnchorValue = 0,
			abscissaValue = 0,
			abscissaRatio=None,
			#
			baseunit='cm',
			abscissaDirection='right',
			ordinateInversion=False,
			ordinateOffset=0, #TODO: whats that?
			goodHorizontalTickDistance = 2,
			goodVerticalTickDistance = 0.8,
			#
			ordinateTickDivider=1,
			ordinateTickDividerOffset=0,
			skipOrdinateTickOnAbscissa=True,
			abscissaTickDivider=1,
			abscissaTickDividerOffset=0,
			skipAbscissaTickOnOrdinate=True,
			#
			ordinateTickLabelDivider=1,
			ordinateTickLabelDividerOffset=0,
			skipOrdinateTickLabelOnAbscissa=None,
			abscissaTickLabelDivider=1,
			abscissaTickLabelDividerOffset=0,
			skipAbscissaTickLabelOnOrdinate=None,
			#
			ordinateJutLength=0.5,
			abscissaJutLength=0.5,
			#
			ordinateTickNegativeLength=0.1,
			ordinateTickPositiveLength=0.1,
			abscissaTickNegativeLength=0.1,
			abscissaTickPositiveLength=0.1,
			#
			ordinateHelpLineDivider=1,
			ordinateHelpLineDividerOffset=0,
			skipOrdinateHelpLineOnAbscissa=True,
			abscissaHelpLineDivider=1,
			abscissaHelpLineDividerOffset=0,
			skipAbscissaHelpLineOnOrdinate=True,
			#
			ordinateHelpLineStyle='catsvOrdinateHelpLine',
			abscissaHelpLineStyle='catsvAbscissaHelpLine',
			#
			bodyStyle=u'catsvBody',
			ordinateStyle=u'catsvOrdinate',
			abscissaStyle=u'catsvAbscissa',
			ordinateJutStyle=u'catsvOrdinateJut',
			abscissaJutStyle=u'catsvAbscissaJut',
			ordinateTickStyle=u'catsvOrdinateTick',
			abscissaTickStyle=u'catsvAbscissaTick',
			#
			ordinateTickLabelStyle=['catsvTickLabelLeft','catsvTickLabelRight','catsvTickLabelBelow','catsvTickLabelAbove'],
			abscissaTickLabelStyle=[None,'catsvTickLabelBelow','catsvTickLabelLeft','catsvTickLabelAbove','catsvTickLabelRight',],
			outerOrdinateLabelStyle=['catsvOuterAxisLabel1','catsvOuterAxisLabel2','catsvOuterAxisLabel3','catsvOuterAxisLabel4','catsvOuterAxisLabel5','catsvOuterAxisLabel6','catsvOuterAxisLabel7','catsvOuterAxisLabel8',],
			outerAbscissaLabelStyle=['catsvOuterAxisLabel3','catsvOuterAxisLabel7','catsvOuterAxisLabel1','catsvOuterAxisLabel5','catsvOuterAxisLabel4','catsvOuterAxisLabel8','catsvOuterAxisLabel2','catsvOuterAxisLabel6',],
			alongsideOrdinateLabelStyle=['catsvAlongsideLabelRight','catsvAlongsideLabelLeft','catsvAlongsideLabelBelow','catsvAlongsideLabelAbove',],
			alongsideAbscissaLabelStyle=[None,'catsvAlongsideLabelBelow','catsvAlongsideLabelRight','catsvAlongsideLabelAbove','catsvAlongsideLabelLeft',],
			orientedLabelStyleAbove=[None,'catsvOrientedLabelAbove','catsvOrientedLabelRight','catsvOrientedLabelBelow','catsvOrientedLabelLeft',],
			orientedLabelStyleBelow=[None,'catsvOrientedLabelBelow','catsvOrientedLabelLeft','catsvOrientedLabelAbove','catsvOrientedLabelRight',],
			orientedLabelStyleLeft=[None,'catsvOrientedLabelLeft','catsvOrientedLabelAbove','catsvOrientedLabelRight','catsvOrientedLabelBelow',],
			orientedLabelStyleRight=[None,'catsvOrientedLabelRight','catsvOrientedLabelBelow','catsvOrientedLabelLeft','catsvOrientedLabelAbove',],
			orientedLabelStyleCenter='catsvOrientedLabelCenter',
			#
			alongsideOrdinateLabelPosition=None,
			alongsideAbscissaLabelPosition=None,
			#
			outerOrdinateLabelString=u"",
			outerAbscissaLabelString=u"",
			alongsideOrdinateLabelString=u"",
			alongsideAbscissaLabelString=u"",
			ordinateTickLabelString=u'{v}',
			abscissaTickLabelString=u'{v}',
			#
			tdhInteger_10Thres = 7,
			tdhInteger_5Thres = 4.2,
			tdhInteger_2Thres = 1.5,
		)
	simple_preset = Attributes(
			parent = empty_preset,
			#
			ordinateLength=5,
			ordinateTickDistance=None,
			ordinateMin=0,
			ordinateMax=10,
			ordinateDomain=None,
			ordinateTickCount=None,
			ordinateValue=0,
			ordinateRatio=None,
			#
			abscissaLength=10,
			abscissaTickDistance=None,
			abscissaMin=0,
			abscissaMax=20,
			abscissaDomain=None,
			abscissaTickCount=None,
			abscissaValue=0,
			abscissaRatio=None,
		)
	presettings =  {
		'empty': empty_preset,
		'simple': simple_preset
	}
			
			
	def __init__(self,preset="empty", printComments=True, **kwargs):
		self.attributes = TikZCoordinateSystem.presettings[preset].copy()
		self.attributes.extend(**kwargs)
		self.printComments = printComments
		self.abscissaDefined = False # will be set to true by the getTikZ-Method if the abscissa is in visible range
		self.ordinateDefined = False # will be set to true by the getTikZ-Method if the ordinate is in visible range
		#
		self.attributeDict = self.attributes.getDict()
		##
		##
	def coord(self,c):
		return self.transformCoord(self.realToTikzX(c[0]),self.realToTikzY(c[1]))
	def realToTikzX(self,x):
		return x*self.attributes.abscissaRatio
	def realToTikzY(self,y):
		return y*self.attributes.ordinateRatio
	def tikzToRealX(self,x):
		return x/self.attributes.abscissaRatio
	def tikzToRealY(self,y):
		return y/self.attributes.abscissaRatio
		##
	def tcoord(self,c):
		k= self.coord(c)
		result = "(%f%s,%f%s)"% (k[0],self.attributes.baseunit,k[1],self.attributes.baseunit)
		return result
		##
	def transformTikZVector(self,x,y):
		if self.attributes.abscissaDirection == 'right':
			return (x,-y) if self.attributes.ordinateInversion else (x,y)
		if self.attributes.abscissaDirection == 'left':
			return (-x,-y) if self.attributes.ordinateInversion else (-x,y)
		if self.attributes.abscissaDirection == 'top':
			return (y,-x) if self.attributes.ordinateInversion else (y,x)
		if self.attributes.abscissaDirection == 'down':
			return (-y,-x) if self.attributes.ordinateInversion else (-y,x)
		##
	#TODO: low: getStyle should always add an additional "additionalStyle" if its present :)
	#TODO: low: give this method a better error handling and API docs!
	def getStyle(self, style):
		styleEntry = self.attributeDict[style]
		if not isinstance(styleEntry, list):
			return styleEntry
		l=len(styleEntry)
		if l == 1:
			return styleEntry[0]
		if l == 2:
			return styleEntry[0] if self.attributes.abscissaDirection=='right' or self.attributes.abscissaDirection=='left' else styleEntry[1]
		##
		i = -1
		if self.attributes.abscissaDirection=='right': i=0
		if self.attributes.abscissaDirection=='left': i=1
		if self.attributes.abscissaDirection=='top': i=2
		if self.attributes.abscissaDirection=='down': i=3
		if i < 0:
			raise ziktRenderException("unknown abscissa direction")
		##
		if l == 4:
			return styleEntry[i]
		if l == 5:
			if self.attributes.ordinateInversion:
				i+=4
			i=int(i/2)+1
			return styleEntry[i]
		if l == 8:
			if self.attributes.ordinateInversion:
				i+=4
			return styleEntry[i]
		##
		##
	@staticmethod	
	def axisDistribution(delta, anchorValue, minValue, maxValue):
		offset = (anchorValue - minValue) % delta
		if offset < 0:
			offset += delta
		if offset < 0:
			raise "Algorithm hurts assumption. Debugging necessary."
		value = minValue + offset
		while value <= maxValue:
			yield round(value,3)
			value += delta
		##
		##
	def getAxisString(
					self,
					minValue,
					maxValue,
					axisValue,
					orthogonalMinValue,
					orthogonalMaxValue,
					orthogonalAxisValue,
					tickDelta,
					tickAnchorValue,
					tickStyle,
					helpLineStyle,
					helpLineDivider,
					helpLineDividerOffset,
					tickLabelStyle,
					tickLabelString,
					tickLabelDivider,
					tickLabelDividerOffset,
					skipTickOnOrthogonalAxis,
					skipHelpLineOnOrthogonalAxis,
					skipTickLabelOnOrthogonalAxis,
					tickCoordToNegativeVector,
					tickNegativeToPositiveVector,
					coordinateMapFunction
			):
		#ordinate ticks, help lines and tick labels
		if tickDelta > 0:
			tickStringArray = []
			tickLabelStringArray = []
			helpLineArray = []
			iterationIndex = 0
			for tickValue in TikZCoordinateSystem.axisDistribution(
				delta = tickDelta,
				anchorValue = tickAnchorValue,
				minValue = minValue,
				maxValue = maxValue
			):
				coordinateMapFunction(tickValue,axisValue)
				tickCoordinate = self.tcoord(coordinateMapFunction(tickValue,axisValue))
				#help line
				if helpLineStyle != None and helpLineDivider != 0:
					if (
						(helpLineDivider < 0 or (iterationIndex+helpLineDividerOffset) % ceil(helpLineDivider) == 0) 
						and
						(not skipHelpLineOnOrthogonalAxis or tickValue != orthogonalAxisValue)
						):
						if  helpLineDivider < 0:
							c = -helpLineDivider
						else:
							c = 1
						lineDistance = tickDelta / c
						for i in range(c):
							a = coordinateMapFunction(orthogonalMinValue,tickValue-i*lineDistance)
							b = coordinateMapFunction(orthogonalMaxValue,tickValue-i*lineDistance)
							helpLineArray.append("\\path[{helpLineStyle}] {start} -- {end};".format(**{
								'start':self.tcoord((a[1],a[0])),
								'end':self.tcoord((b[1],b[0])),
								'helpLineStyle':helpLineStyle
							}))
				#tick
				if tickValue != orthogonalAxisValue or not skipTickOnOrthogonalAxis:
					tickStringArray.append(
						"\\path[{tickStyle}] {c}  ++{v1} -- ++{v2};".format(					
							c=tickCoordinate,
							v1=tickCoordToNegativeVector,
							v2=tickNegativeToPositiveVector,
							tickStyle=tickStyle
						)
					)
				# tick label
				if ((iterationIndex + tickLabelDividerOffset) % tickLabelDivider == 0) and (not skipTickLabelOnOrthogonalAxis or tickValue != orthogonalAxisValue):
					v=int(tickValue) if tickValue%1==0 else tickValue 
					tickLabelStringArray.append("\\path{c} node [{tickLabelStyle}] {{{label}}};".format(**{
						'c': tickCoordinate,
						'label':(tickLabelString.format(v=v, value=v)),
						'tickLabelStyle':tickLabelStyle
					}))
				iterationIndex += 1
				##
			result = []
			if helpLineStyle != None:
				result.append('\n'.join(helpLineArray))
			if tickStyle != None:
				result.append('\n'.join(tickStringArray))
			result.append('\n'.join(tickLabelStringArray))
			return '\n'.join(result)
			##
		else:
			return ''
		##
		##
	#TODO: low: write test cases for this method
	@staticmethod
	def initializeDimension(minValue, maxValue, domain, length, ratio, forcedMin = None, forcedMax = None):
		if domain != None and (not isinstance(domain, int) and not isinstance(domain,float)):
			raise ziktRenderException("domain value must be a number (integer or float)")
		if length != None and (not isinstance(length, int) and not isinstance(length,float)):
			raise ziktRenderException("domain value must be a number (integer or float)")
		if ratio != None and (not isinstance(ratio, int) and not isinstance(ratio,float)):
			raise ziktRenderException("domain value must be a number (integer or float)")
		##
		if ratio == None and length == None:
			raise ziktRenderException("ratio or length must be specified")
		if ratio != None and length != None:
			derivedDomain = length / ratio
			if domain == None:
				domain = derivedDomain
			else:
				if domain != derivedDomain:
					raise ziktRenderException(u"Length, ratio and domain are specified but length ≠ ratio × domain so the ratio is overspecified. Set one of them to None.")
		dataDomainIntervallConstraints = 0
		if minValue != None: dataDomainIntervallConstraints += 1
		if maxValue != None: dataDomainIntervallConstraints += 1
		if domain != None: dataDomainIntervallConstraints += 1
		if dataDomainIntervallConstraints < 2:
			raise ziktRenderException("Domain interval underspecified.")
		if dataDomainIntervallConstraints > 2:
			raise ziktRenderException("Domain interval overspecified.")
		if minValue == None:
			minValue = maxValue - domain
		if maxValue == None:
			maxValue = minValue + domain
		if forcedMin != None:
			minValue = min(forcedMin,minValue)
		if forcedMax != None:
			maxValue = max(forcedMax,maxValue)
		domain = maxValue - minValue
		if domain <= 0:
			raise ziktRenderException("Minimum and maximum value defines a negative or empty domain.")
			##
		if length == None:
			length = domain * ratio
		if ratio == None:
			ratio = length / domain
		if length <= 0:
			raise ziktRenderException("Length is negative or zero.")
		return minValue, maxValue, domain, length, ratio
		##
		##
	def initializeAxisDecoration(self, tickDelta, tickAnchorValue, helpLineDivider, helpLineDividerOffset, labelDivider, labelDividerOffset, minValue, maxValue, domain, length, axisIsHorizontal):
		if tickDelta == None:
			# tick delta is None, so we use a little heuristic algorithm to calculate it
			if axisIsHorizontal:
				goodTickDistance = self.attributes.goodHorizontalTickDistance
			else:
				goodTickDistance = self.attributes.goodVerticalTickDistance
			goodTickCount = length / goodTickDistance
			goodTickDelta = domain / goodTickCount
			decimalDimension = floor(log(goodTickDelta, 10))
			densityKey = goodTickDelta * pow(10,-decimalDimension)
			#TODO: low: in tick distribution heuristic: serve an optional fraction of [0,1/4,2/4,3/4,1] (and make it default for decimal dimensions < 0)
			if densityKey < self.attributes.tdhInteger_2Thres:
				multiplier = 1
			elif densityKey > self.attributes.tdhInteger_10Thres:
				multiplier = 10
			elif densityKey > self.attributes.tdhInteger_5Thres:
				multiplier = 5
			else:
				multiplier = 2
			tickDelta = pow(10,(decimalDimension)) * multiplier
		return tickDelta, tickAnchorValue, helpLineDivider, helpLineDividerOffset, labelDivider, labelDividerOffset
		##
		##
	def getTikZ(self):
		# initialize ordinate
		try:
			(self.attributes.ordinateMin,
			self.attributes.ordinateMax,
			self.attributes.ordinateDomain,
			self.attributes.ordinateLength,
			self.attributes.ordinateRatio) = TikZCoordinateSystem.initializeDimension(
		            self.attributes.ordinateMin,
		            self.attributes.ordinateMax,
		            self.attributes.ordinateDomain,
		            self.attributes.ordinateLength,
		            self.attributes.ordinateRatio,
		            forcedMin = self.attributes.ordinateForcedDisplayMin,
		            forcedMax = self.attributes.ordinateForcedDisplayMax,
		        )
		except Exception as e:
			logging.error("Initializing ordinate failed: " + e.message)
		# initialize abscissa
		try:
			(self.attributes.abscissaMin,
			self.attributes.abscissaMax,
			self.attributes.abscissaDomain,
			self.attributes.abscissaLength,
			self.attributes.abscissaRatio) = TikZCoordinateSystem.initializeDimension(
		            self.attributes.abscissaMin, self.attributes.abscissaMax, self.attributes.abscissaDomain, self.attributes.abscissaLength, self.attributes.abscissaRatio
		        )
		except Exception as e:
			logging.error("Initializing abscissa failed: " + e.message)
		# initialize ordinate decoration
		(
		self.attributes.ordinateTickDelta,
		self.attributes.ordinateTickAnchorValue,
		self.attributes.ordinateHelpLineDivider,
		self.attributes.ordinateHelpLineDividerOffset,
		self.attributes.ordinateTickLabelDivider,
		self.attributes.ordinateTickLabelDividerOffset
		) = self.initializeAxisDecoration(
			tickDelta = self.attributes.ordinateTickDelta,
			tickAnchorValue = self.attributes.ordinateTickAnchorValue,
			helpLineDivider = self.attributes.ordinateHelpLineDivider,
			helpLineDividerOffset = self.attributes.ordinateHelpLineDividerOffset,
			labelDivider = self.attributes.ordinateTickLabelDivider,
			labelDividerOffset = self.attributes.ordinateTickLabelDividerOffset,
			minValue = self.attributes.ordinateMin,
			maxValue = self.attributes.ordinateMax,
			domain = self.attributes.ordinateDomain,
			length = self.attributes.ordinateLength,
			axisIsHorizontal = self.attributes.abscissaDirection == 'top' or self.attributes.abscissaDirection == 'down'
		)
		# initialize abscissa decoration
		(
		self.attributes.abscissaTickDelta,
		self.attributes.abscissaTickAnchorValue,
		self.attributes.abscissaHelpLineDivider,
		self.attributes.abscissaHelpLineDividerOffset,
		self.attributes.abscissaTickLabelDivider,
		self.attributes.abscissaTickLabelDividerOffset
		) = self.initializeAxisDecoration(
			tickDelta = self.attributes.abscissaTickDelta,
			tickAnchorValue = self.attributes.abscissaTickAnchorValue,
			helpLineDivider = self.attributes.abscissaHelpLineDivider,
			helpLineDividerOffset = self.attributes.abscissaHelpLineDividerOffset,
			labelDivider = self.attributes.abscissaTickLabelDivider,
			labelDividerOffset = self.attributes.abscissaTickLabelDividerOffset,
			minValue = self.attributes.abscissaMin,
			maxValue = self.attributes.abscissaMax,
			domain = self.attributes.abscissaDomain,
			length = self.attributes.abscissaLength,
			axisIsHorizontal = self.attributes.abscissaDirection == 'left' or self.attributes.abscissaDirection == 'right'
		)
		# initialize tick skips
		if self.attributes.skipOrdinateTickLabelOnAbscissa == None:
			if self.attributes.ordinateValue >  self.attributes.abscissaMin:
				self.attributes.skipOrdinateTickLabelOnAbscissa = True
			else:
				self.attributes.skipOrdinateTickLabelOnAbscissa = False
		if self.attributes.skipAbscissaTickLabelOnOrdinate == None:
			if self.attributes.abscissaValue >  self.attributes.ordinateMin:
				self.attributes.skipAbscissaTickLabelOnOrdinate = True
			else:
				self.attributes.skipAbscissaTickLabelOnOrdinate = False
		# initialize the array that will be used to add output strings; the fields will be
		#  concatenated (separated by \n) at the end of this method to form the resulting TikZ string  
		result=[]
		#body
		if self.getStyle("bodyStyle") != None:
			result.append(
				"\\path[{bodyStyle}] {start} rectangle {end};".format(**{
					'start':self.tcoord((self.attributes.abscissaMin,self.attributes.ordinateMin)),
					'end':self.tcoord((self.attributes.abscissaMax,self.attributes.ordinateMax)),
					'bodyStyle':self.getStyle("bodyStyle")
				}))
		#ORDINATE
		if (self.attributes.ordinateValue >= self.attributes.abscissaMin) and (self.attributes.ordinateValue <= self.attributes.abscissaMax):
			self.ordinateDefined = True
			#ordinate coordinates
			result.append("\\path[coordinate]\n\t{ordinatestart} coordinate (ordinateStart)\n\t{ordinateend} coordinate (ordinateEnd);".format(**{
					'ordinatestart':self.tcoord((self.attributes.ordinateValue,self.attributes.ordinateMin)),
					'ordinateend':self.tcoord((self.attributes.ordinateValue,self.attributes.ordinateMax))
				}))
			# help lines, ticks, tick labels
			result.append(self.getAxisString(
				minValue = self.attributes.ordinateMin,
				maxValue = self.attributes.ordinateMax,
				axisValue = self.attributes.ordinateValue,
				orthogonalMinValue = self.attributes.abscissaMin,
				orthogonalMaxValue = self.attributes.abscissaMax,
				orthogonalAxisValue = self.attributes.abscissaValue,
				tickDelta = self.attributes.ordinateTickDelta,
				tickAnchorValue = self.attributes.ordinateTickAnchorValue,
				tickStyle = self.attributes.ordinateTickStyle,
				helpLineStyle = self.attributes.ordinateHelpLineStyle,
				helpLineDivider = self.attributes.ordinateHelpLineDivider,
				helpLineDividerOffset = self.attributes.ordinateHelpLineDividerOffset,
				tickLabelStyle = self.getStyle('ordinateTickLabelStyle'),
				tickLabelString = self.attributes.ordinateTickLabelString,
				tickLabelDivider = self.attributes.ordinateTickLabelDivider,
				tickLabelDividerOffset = self.attributes.ordinateTickLabelDividerOffset,
				skipTickOnOrthogonalAxis = self.attributes.skipOrdinateTickOnAbscissa,
				skipTickLabelOnOrthogonalAxis = self.attributes.skipOrdinateTickLabelOnAbscissa,
				skipHelpLineOnOrthogonalAxis = self.attributes.skipOrdinateHelpLineOnAbscissa,
				tickCoordToNegativeVector = self.transformTikZVector(-self.attributes.ordinateTickNegativeLength,0),
				tickNegativeToPositiveVector = self.transformTikZVector(self.attributes.ordinateTickNegativeLength + self.attributes.ordinateTickPositiveLength,0),
				coordinateMapFunction = (lambda a,x: (x,a))
			))
			#ordinate line
			if self.attributes.ordinateStyle != None:
				result.append("\path [{ordinateStyle}] (ordinateStart) -- (ordinateEnd);".format(ordinateStyle=self.getStyle('ordinateStyle')))
			#ordinate jut and its coordinate
			if self.attributes.ordinateJutStyle != None:
				vector=self.transformCoord(0,self.attributes.ordinateJutLength)
				jutend=self.coord((self.attributes.ordinateValue,self.attributes.ordinateMax))+vector
				result.append("\\path[coordinate] {jutend} coordinate (ordinateJut);".format(jutend=jutend))
				result.append("\\path[{ordinateJutStyle}] (ordinateEnd) -- (ordinateJut);".format(ordinateJutStyle=self.getStyle('ordinateJutStyle')))
			#outer ordinate label
			if self.attributes.outerOrdinateLabelString != "" and self.attributes.outerOrdinateLabelStyle != None:
				result.append("\t\\path (ordinateEnd) node[{outerOrdinateLabelStyle}] {{{label}}};".format(**{
					'outerOrdinateLabelStyle':self.getStyle('outerOrdinateLabelStyle'),
					'label':self.attributes.outerOrdinateLabelString.format(**self.attributeDict)
				}))
			#alongside ordinate label
			if self.attributes.alongsideOrdinateLabelString != "" and self.attributes.alongsideOrdinateLabelStyle != None:
				y=self.attributes.alongsideOrdinateLabelPosition
				if y == None:
					if self.attributes.abscissaValue <= self.attributes.ordinateMin:
						y=self.attributes.ordinateMin + self.attributes.ordinateDomain * 3 / 5
					else:
						y=((self.attributes.ordinateMin + (self.attributes.abscissaValue - self.attributes.ordinateMin) / 3) if abs(self.attributes.abscissaValue-self.attributes.ordinateMin) > abs(self.attributes.abscissaValue - self.attributes.ordinateMax) else self.attributes.abscissaValue + (self.attributes.ordinateMax - self.attributes.abscissaValue)*2 / 3)
				result.append("\t\\path {c} node[{alongsideOrdinateLabelStyle}] {{{label}}};".format(**{
					'c':self.tcoord((self.attributes.ordinateValue,y)),
					'alongsideOrdinateLabelStyle':self.getStyle('alongsideOrdinateLabelStyle'),
					'label':self.attributes.alongsideOrdinateLabelString.format(**self.attributeDict)
				}))
		#ORDINATE＿END
		#ABSCISSA
		if (self.attributes.abscissaValue >= self.attributes.ordinateMin) and (self.attributes.abscissaValue <= self.attributes.ordinateMax):
			self.abscissaDefined = True
			#abscissa coordinates
			result.append("\\path[coordinate]\n\t{abscissastart} coordinate (abscissaStart)\n\t{abscissaend} coordinate (abscissaEnd)\n;".format(**{
					'abscissastart':self.tcoord((self.attributes.abscissaMin,self.attributes.abscissaValue)),
					'abscissaend':self.tcoord((self.attributes.abscissaMax,self.attributes.abscissaValue))
				}))
			# help lines, ticks, tick labels
			result.append(self.getAxisString(
				minValue = self.attributes.abscissaMin,
				maxValue = self.attributes.abscissaMax,
				axisValue = self.attributes.abscissaValue,
				orthogonalMinValue = self.attributes.ordinateMin,
				orthogonalMaxValue = self.attributes.ordinateMax,
				orthogonalAxisValue = self.attributes.ordinateValue,
				tickDelta = self.attributes.abscissaTickDelta,
				tickAnchorValue = self.attributes.abscissaTickAnchorValue,
				tickStyle = self.attributes.abscissaTickStyle,
				helpLineStyle = self.attributes.abscissaHelpLineStyle,
				helpLineDivider = self.attributes.abscissaHelpLineDivider,
				helpLineDividerOffset = self.attributes.abscissaHelpLineDividerOffset,
				tickLabelStyle = self.getStyle('abscissaTickLabelStyle'),
				tickLabelString = self.attributes.abscissaTickLabelString,
				tickLabelDivider = self.attributes.abscissaTickLabelDivider,
				tickLabelDividerOffset = self.attributes.abscissaTickLabelDividerOffset,
				skipTickOnOrthogonalAxis = self.attributes.skipAbscissaTickOnOrdinate,
				skipTickLabelOnOrthogonalAxis = self.attributes.skipAbscissaTickLabelOnOrdinate,
				skipHelpLineOnOrthogonalAxis = self.attributes.skipAbscissaHelpLineOnOrdinate,
				tickCoordToNegativeVector = self.transformTikZVector(0,-self.attributes.abscissaTickNegativeLength),
				tickNegativeToPositiveVector = self.transformTikZVector(0,self.attributes.abscissaTickNegativeLength + self.attributes.abscissaTickPositiveLength),
				coordinateMapFunction = (lambda a,x: (a,x))
			))
			#abscissa line
			if self.attributes.abscissaStyle != None:
				result.append("\path [{abscissaStyle}] (abscissaStart) -- (abscissaEnd);".format(abscissaStyle=self.getStyle('abscissaStyle')))		
			#abscissa jut and its coordinate
			if self.attributes.abscissaJutStyle != None:
				vector=self.transformCoord(self.attributes.abscissaJutLength,0)
				jutend=self.coord((self.attributes.abscissaMax,self.attributes.abscissaValue))+vector
				result.append("\\path[coordinate] {jutend} coordinate (abscissaJut);".format(jutend=jutend))
				result.append("\\path[{abscissaJutStyle}] (abscissaEnd) -- (abscissaJut);".format(abscissaJutStyle=self.getStyle('abscissaJutStyle')))
			#outer abscissa label
			if self.attributes.outerAbscissaLabelString != "" and self.attributes.outerAbscissaLabelStyle != None:
				result.append("\t\\path (abscissaEnd) node[{outerAbscissaLabelStyle}] {{{label}}};".format(**{
					'outerAbscissaLabelStyle':self.getStyle('outerAbscissaLabelStyle'),
					'label':self.attributes.outerAbscissaLabelString.format(**self.attributeDict)
				}))
			#alongside abscissa label
			if self.attributes.alongsideAbscissaLabelString != "" and self.attributes.alongsideAbscissaLabelStyle != None:
				x=self.attributes.alongsideAbscissaLabelPosition
				if x == None:
					if self.attributes.ordinateValue <= self.attributes.abscissaMin:
						x=self.attributes.abscissaMin + self.attributes.abscissaDomain * 3/5
					else:
						x=((self.attributes.abscissaMin + (self.attributes.ordinateValue - self.attributes.abscissaMin) / 3) if abs(self.attributes.ordinateValue-self.attributes.abscissaMin) > abs(self.attributes.ordinateValue - self.attributes.abscissaMax) else self.attributes.ordinateValue + (self.attributes.abscissaMax - self.attributes.ordinateValue) *2/3)
				result.append("\t\\path {c} node[{alongsideAbscissaLabelStyle}] {{{label}}};".format(**{
					'c':self.tcoord((x,self.attributes.abscissaValue)),
					'alongsideAbscissaLabelStyle':self.getStyle('alongsideAbscissaLabelStyle'),
					'label':self.attributes.alongsideAbscissaLabelString.format(**self.attributeDict)
				}))
		#ABSCISSA＿END
		return "\n".join(result)
		##
		##
	def drawLine(self,start,end,style):
		return "\\path[{style}] {start} -- {end};".format(style=style,start=self.tcoord(start),end=self.tcoord(end))
		##
		##
	def drawRectangle(self,start,end,style):
		return "\\path[{style}] {start} rectangle {end};".format(style=style,start=self.tcoord(start),end=self.tcoord(end))
		##
		##
	def transformCoord(self,x,y):
		if self.attributes.abscissaDirection == 'right':
			return Coord([x,y if not self.attributes.ordinateInversion else -y])
		if self.attributes.abscissaDirection == 'left':
			return Coord([-x,y if not self.attributes.ordinateInversion else -y])
		if self.attributes.abscissaDirection == 'top':
			return Coord([y if not self.attributes.ordinateInversion else -y,x])
		if self.attributes.abscissaDirection == 'down':
			return Coord([y if not self.attributes.ordinateInversion else -y,-x])
		raise ziktRenderException( "unknown abscissa direction")
		##
		##
	def render(self,table):
		return self.getTikZ()
		##
		##
#	def configureOrdinateByAxisHeuristic(self, heuristicClass):
#		heuristic = heuristicClass(
#			minValue=self.attributes.minOrdinateValue,
#			maxValue=self.attributes.maxOrdinateValue,
#			length=self.attributes.ordinateLength,
#			attributesDict = self.attributes.getDict()
#		)
	def configureOrdinateByAxisHeuristic(self, heuristic):
		if self.ordinateSpecificationState() < 0:
			self.attributes.ordinateMin = heuristic.getMinValue()
			self.attributes.ordinateMax = heuristic. getMaxValue()
			self.attributes.ordinateDomain = None
		if self.attributes.ordinateTickDelta == None:
			self.attributes.ordinateTickDelta = heuristic. getTickValueDistance()
		
		
		#sys.stderr.write("CONFIGURE min: %s   max: %s   domain: %s\n" %(str(self.attributes.ordinateMin), str(self.attributes.ordinateMax), str(self.attributes.ordinateDomain)))
		#this.attributes.??? = heuristic. getTickValueOffset()
		#this.attributes.ordinateMin = heuristic. getTickLabelValueDistance()
		#this.attributes.ordinateMin = heuristic. getTickLabelValueOffset()
if __name__ == "__main__":
	c =  TikZCoordinateSystem(
		preset='simple',
		ordinateLength=2,
		abscissaLength=12,
		ordinateMin=-1,
		abscissaMin=-1,
		ordinateMax=1,
		abscissaMax=1,
		)
	print c.render(FloatTable())

#c = TikZCoordinateSystem(preset='simple')
#c.render(FloatTable())

#print c.getTikZ()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''


from distutils.core import setup

try:
	with open ("about", "r") as aboutfile:
		about = aboutfile.read()
except:
	about = ""

setup(
	name='ZikT',
	version='0.1',
	description='Python lib for generating TikZ charts from table data',
	author='Daniel Llin Ferrero',
	author_email='texnh@llin.info',
	url='https://pypi.python.org/pypi/zikt/',
	package_dir={'': 'src'},
	packages=['zikt'],
	package_data={'zikt': ['doc/zikt.pdf','latex/zikt.sty']},
	scripts=['scripts/zikt'],
	requires=['unicodecsv'],
	license='LGPL 3  (http://www.gnu.org/licenses/lgpl-3.0)',
	classifiers=[
		'Development Status :: 2 - Pre-Alpha',
		'Environment :: Console',
		'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
	],
	long_description = about
)



# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

class Legend():
	##
	def __init__(self, table, attributes):
		self.table = table
		self.attributes = attributes
		self.basicLegendItemStyle = "basiclegenditem"
		##
		##
	def getRowAttribute(self, rowIndex, attributeName):
# 		if self.attributes.stackmode and self.attributes.stackmode == "onTopOfFirstRow":
# 			rowIndex -= 1
		attribute = self.attributes.getDict()[attributeName]
		if not isinstance(attribute, list):
			return attribute
		else:
			return attribute[rowIndex % len(attribute)]
		##
		##
	def get_tikz(self,anchor):
		result = []
		rowCount = 1
		if self.attributes.stackmode and self.attributes.stackmode == "onTopOfFirstRow":
			rowHeaders = self.table.getRowHeaders()[1:]
		else:
			rowHeaders = self.table.getRowHeaders()
		for rowHeader in rowHeaders:
			style = self.attributes.legendItemStyle if hasattr(self.attributes, 'legendItemStyle') else self.basicLegendItemStyle
			name = "legend"+str(rowCount)
			isVertical = True if hasattr(self.attributes, 'legendDirection') and self.attributes.legendDirection == "Vertical" else False
			style += (",right=of (%s)" if isVertical else ",below=of (%s)")%(anchor) 
			result.append("\\path ({c}) node[{style}] ({name}) {{{label}}};".format(c=anchor, style=style, name = name, label=rowHeader))
			anchor=name
			rowCount += 1
		
	def get_hacked_floating_legend(self):
		index = 0
		result=[]
		if self.attributes.stackmode and self.attributes.stackmode == "onTopOfFirstRow":
			rowHeaders = self.table.getRowHeaders()[1:]
		else:
			rowHeaders = self.table.getRowHeaders()
		for rowHeader in rowHeaders:
			draw = self.getRowAttribute(index,"drawColors")
			fill = self.getRowAttribute(index,"fillColors")
			legendFont = self.getRowAttribute(index, "legendFont")
			index += 1
			if legendFont == None or legendFont == "":
				legendfontstyle = ""
			else:
				legendfontstyle = "font="+legendFont
			result.append("\\tikz{")
			result.append("\\coordinate(a);\\path[ fill=%s, draw=%s,ziktBaseLine] (a) ++(-1mm,0mm) -- ++(0mm,1mm) -- ++(2mm,0mm) -- ++(0mm,-2mm) -- ++(-2mm,0mm) -- cycle;"%(fill,draw))
			result.append("\\node[right=1mm of a.east, anchor=west,text width=,text depth=-0.1ex,%s]{%s};"%(legendfontstyle, rowHeader))
			result.append('}')
		return '\n'.join(result)  


# -*- coding: utf-8 -*-
# This file is part of the Python library ZikT.
# 
# ZikT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License,
# version 3, as published by the Free Software Foundation.
# 
# ZikT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with ZikT. If not, see <http://www.gnu.org/licenses/>.
'''
@author: Daniel Llin Ferrero
'''

import unittest
from tables import Table, FloatTable, StringTable
from helper import ziktTableException

def getBasicTable():
	table = Table()
	table.addRow(["","A", "B", "C", "D", "E", "F"])
	table.addRow(["α","1", "2", "3", "4", "5"])
	table.addRow(["β","11", "12", "13", "14", "15", "16"])
	table.addRow(["γ","21", "22", "23"])
	return table

class TableTests(unittest.TestCase):
	
	def test_getColCount(self):
		table = getBasicTable()
		self.failUnlessEqual(table.getColCount(),6)
		
	def test_getRowCount(self):
		table = getBasicTable()
		self.failUnlessEqual(table.getRowCount(),3)
		
	def test_getMinRowLength(self):
		table = getBasicTable()
		self.failUnlessEqual(table.getMinRowLength(), 3)
		
	def test_getMaxRowLength(self):
		table = getBasicTable()
		self.failUnlessEqual(table.getMaxRowLength(), 6)
		
	def test_getCell(self):
		table = getBasicTable()
		self.failUnlessEqual(table.getCell(3, 0), '4')
		
	def test_setCell_cell_exists(self):
		table = getBasicTable()
		self.assertEqual(table.getCell(2, 2), '23')
		table.setCell(2, 2, "Wurst")
		self.assertEqual(table.getCell(2, 2), 'Wurst')
		
	def test_setCell_cell_does_not_exist(self):
		table = getBasicTable()
		table.setCell(4, 2, "Wurst")
		self.assertEqual(table.getCell(3, 2), None)
		self.assertEqual(table.getCell(4, 2), 'Wurst')	
		
	def test_getCell_exceptionOnColOutOfRange(self):
		table = getBasicTable()
		self.failUnlessRaises(ziktTableException, table.getCell, 6, 0)
		
	def test_getCell_exceptionOnRowOutOfRange(self):
		table = getBasicTable()
		self.failUnlessRaises(ziktTableException, table.getCell, 0, 4)
		
	def test_swap(self):
		table = getBasicTable()
		self.failUnlessEqual(table.getCell(1, 2), '22')
		self.failUnlessEqual(table.getCell(2, 1), '13')
		table.swap()
		self.failUnlessEqual(table.getCell(1, 2), '13')
		self.failUnlessEqual(table.getCell(2, 1), '22')
		
	def test_unfoldRow(self):
		row = ["header","a:1,b:2","c:3,d:4"]
		table = StringTable()
		table.hasColHeader=False
		table.addRow(row)
		table.unfoldRow(rowIndex = 0, fillValue="0",line_delimiter=',',tuple_delimiter=':')
		#
		self.assertEqual(table.getRowCount(),4)
		self.assertEqual(table.getRowHeader(0),"a")
		self.assertEqual(table.getRowHeader(1),"b")
		self.assertEqual(table.getRowHeader(2),"c")
		self.assertEqual(table.getRowHeader(3),"d")
		self.assertEqual(table.getCell(0,0),"1")
		self.assertEqual(table.getCell(0,1),"2")
		self.assertEqual(table.getCell(0,2),"0")
		self.assertEqual(table.getCell(0,3),"0")
		self.assertEqual(table.getCell(1,0),"0")
		self.assertEqual(table.getCell(1,1),"0")
		self.assertEqual(table.getCell(1,2),"3")
		self.assertEqual(table.getCell(1,3),"4")


class FloatTableTests(unittest.TestCase):
	
	def test_instantiation(self):
		table = getBasicTable()
		table.addRow(["δ","3", "-2", "3.5", "3"])
		floatTable=FloatTable(table)
		self.failUnlessEqual(floatTable.getCell(2, 3),3.5)
		self.assertEqual(len(table.rowHeader), 4)
		
	def test_instantiation_exceptionOnConvert(self):
		table = getBasicTable()
		table.addRow(["3", "a", "3.0"])
		self.failUnlessRaises(ziktTableException, FloatTable, table)
	
	def test_getColCount(self):
		table = FloatTable(getBasicTable())
		self.failUnlessEqual(table.getColCount(),6)
		
	def test_getRowCount(self):
		table = FloatTable(getBasicTable())
		self.failUnlessEqual(table.getRowCount(),3)
		
	def test_getMinRowLength(self):
		table = FloatTable(getBasicTable())
		self.failUnlessEqual(table.getMinRowLength(), 3)
		
	def test_getMaxRowLength(self):
		table = FloatTable(getBasicTable())
		self.failUnlessEqual(table.getMaxRowLength(), 6)
		
	def test_getCell(self):
		table = FloatTable(getBasicTable())
		self.failUnlessEqual(table.getCell(3, 0), 4)
		
	def test_getMinMax(self):
		table = Table()
		table.addRow(['', 'A', 'B', 'C'])
		table.addRow([u'α', 3, 4, 5])
		table.addRow([u'β', 3, 7, 5])
		floatTable = FloatTable(table)
		minV, maxV = floatTable.getMinMax()
		self.failUnlessEqual(minV,3)
		self.failUnlessEqual(maxV,7)
		floatTable.addRow([u'γ', 30, 7, -5])
		minV, maxV = floatTable.getMinMax()
		self.failUnlessEqual(minV,-5)
		self.failUnlessEqual(maxV,30)
		
	def test_getMinMaxOfCol(self):
		table = FloatTable(getBasicTable())
		minV, maxV = table.getMinMaxOfCol(1)
		self.failUnlessEqual(minV, 2)
		self.failUnlessEqual(maxV, 22)
		
	def test_getMinMaxIndicesOfCol(self):
		table = FloatTable(getBasicTable())
		minV, maxV = table.getMinMaxIndicesOfCol(1)
		self.failUnlessEqual(minV, 0)
		self.failUnlessEqual(maxV, 2)
		
	def test_getMinMaxColSum(self):
		table = FloatTable(getBasicTable())
		minV, maxV = table.getMinMaxColSum()
		self.assertEqual(minV, 16)
		self.assertEqual(maxV, 39)
		
	def test_allPositive_positive(self):
		table = FloatTable(getBasicTable())
		self.assertTrue(table.allPositive())
		
	def test_allPositive_negative(self):
		table = FloatTable(getBasicTable())
		table.addRow(["Δ",3,4,-6,20])
		self.assertFalse(table.allPositive())
		
	def test_allPositiveExceptFirstRow_positive1(self):
		table = FloatTable(getBasicTable())
		self.assertTrue(table.allPositiveExceptFirstRow())
		
	def test_allPositiveExceptFirstRow_positive2(self):
		table = FloatTable(getBasicTable())
		table.rows.insert(0, ["Δ",2,-2,20])
		self.assertTrue(table.allPositiveExceptFirstRow())
		
	def test_allPositiveExceptFirstRow_negative(self):
		table = FloatTable(getBasicTable())
		table.addRow(["Δ",3,4,-6,20])
		self.assertFalse(table.allPositiveExceptFirstRow())
	
	def test_getStacked(self):
		table = FloatTable(getBasicTable())
		stackedTable = table.getStacked(onTopOfZero = True)
		self.assertEqual(stackedTable.getCol(0)[0], 0)
		self.assertEqual(stackedTable.getCol(0)[1], 1)
		self.assertEqual(stackedTable.getCol(0)[2], 12)
		self.assertEqual(stackedTable.getCol(0)[3], 33)
		
#	def test_getStackMinMax(self):
#		raise "mööp"
#	
#	def test_getStackMinMaxOfCol(self):
#		raise "mööp"

if __name__ == "__main__": 
	unittest.main()
